@@ -10,14 +10,16 @@ from gradio.blocks import Blocks
 from gradio.components import (
     HTML,
     JSON,
+    AnnotatedImage,
+    Annotatedimage,
     Audio,
     BarPlot,
     Button,
     Carousel,
     Chatbot,
     Checkbox,
-    Checkboxgroup,
     CheckboxGroup,
+    Checkboxgroup,
     Code,
     ColorPicker,
     DataFrame,
@@ -27,8 +29,8 @@ from gradio.components import (
     File,
     Gallery,
     Highlight,
-    Highlightedtext,
     HighlightedText,
+    Highlightedtext,
     Image,
     Interpretation,
     Json,
@@ -54,6 +56,7 @@ from gradio.components import (
 )
 from gradio.events import SelectData
 from gradio.exceptions import Error
+from gradio.external import load
 from gradio.flagging import (
     CSVLogger,
     FlaggingCallback,
@@ -61,9 +64,8 @@ from gradio.flagging import (
     HuggingFaceDatasetSaver,
     SimpleCSVLogger,
 )
-from gradio.helpers import EventData, Progress
+from gradio.helpers import EventData, Progress, make_waveform, skip, update
 from gradio.helpers import create_examples as Examples
-from gradio.helpers import make_waveform, skip, update
 from gradio.interface import Interface, TabbedInterface, close_all
 from gradio.ipython_ext import load_ipython_extension
 from gradio.layouts import Accordion, Box, Column, Group, Row, Tab, TabItem, Tabs
@@ -85,6 +87,7 @@ from gradio.templates import (
     TextArea,
     Webcam,
 )
+from gradio.themes import Base as Theme
 
 current_pkg_version = (
     (pkgutil.get_data(__name__, "version.txt") or b"").decode("ascii").strip()
