@@ -1,3 +1,4 @@
+# type: ignore
 """
 This module defines various classes that can serve as the `input` to an interface. Each class must inherit from
 `InputComponent`, and each class must define a path to its template. All of the subclasses of `InputComponent` are
@@ -19,7 +20,7 @@ class Textbox(components.Textbox):
         placeholder: Optional[str] = None,
         default: str = "",
         numeric: Optional[bool] = False,
-        type: Optional[str] = "str",
+        type: Optional[str] = "text",
         label: Optional[str] = None,
         optional: bool = False,
     ):
@@ -132,7 +133,7 @@ class CheckboxGroup(components.CheckboxGroup):
     def __init__(
         self,
         choices: List[str],
-        default: List[str] = [],
+        default: List[str] | None = None,
         type: str = "value",
         label: Optional[str] = None,
         optional: bool = False,
@@ -145,6 +146,8 @@ class CheckboxGroup(components.CheckboxGroup):
         label (str): component name in interface.
         optional (bool): this parameter is ignored.
         """
+        if default is None:
+            default = []
         warnings.warn(
             "Usage of gradio.inputs is deprecated, and will not be supported in the future, please import your component from gradio.components",
         )
