@@ -13,7 +13,9 @@ import warnings
 import weakref
 from typing import TYPE_CHECKING, Any, Callable, List, Tuple
 
-from gradio import Examples, interpretation, utils
+from gradio_client.documentation import document, set_documentation_group
+
+from gradio import Examples, external, interpretation, utils
 from gradio.blocks import Blocks
 from gradio.components import (
     Button,
@@ -24,13 +26,11 @@ from gradio.components import (
     get_component_instance,
 )
 from gradio.data_classes import InterfaceTypes
-from gradio.documentation import document, set_documentation_group
 from gradio.events import Changeable, Streamable
 from gradio.flagging import CSVLogger, FlaggingCallback, FlagMethod
 from gradio.layouts import Column, Row, Tab, Tabs
 from gradio.pipelines import load_from_pipeline
 from gradio.themes import ThemeClass as Theme
-from gradio.utils import GRADIO_VERSION
 
 set_documentation_group("interface")
 
@@ -56,7 +56,7 @@ class Interface(Blocks):
         demo = gr.Interface(fn=image_classifier, inputs="image", outputs="label")
         demo.launch()
     Demos: hello_world, hello_world_3, gpt_j
-    Guides: quickstart, key_features, sharing_your_app, interface_state, reactive_interfaces, advanced_interface_features, setting_up_a_gradio_demo_for_maximum_performance
+    Guides: quickstart, key-features, sharing-your-app, interface-state, reactive-interfaces, advanced-interface-features, setting-up-a-gradio-demo-for-maximum-performance
     """
 
     # stores references to all currently existing Interface instances
@@ -77,9 +77,10 @@ class Interface(Blocks):
         api_key: str | None = None,
         alias: str | None = None,
         **kwargs,
-    ) -> Interface:
+    ) -> Blocks:
         """
-        Class method that constructs an Interface from a Hugging Face repo. Can accept
+        Warning: this method will be deprecated. Use the equivalent `gradio.load()` instead. This is a class
+        method that constructs a Blocks from a Hugging Face repo. Can accept
         model repos (if src is "models") or Space repos (if src is "spaces"). The input
         and output components are automatically loaded from the repo.
         Parameters:
@@ -89,14 +90,11 @@ class Interface(Blocks):
             alias: optional string used as the name of the loaded model instead of the default name (only applies if loading a Space running Gradio 2.x)
         Returns:
             a Gradio Interface object for the given model
-        Example:
-            import gradio as gr
-            description = "Story generation with GPT"
-            examples = [["An adventurer is approached by a mysterious stranger in the tavern for a new quest."]]
-            demo = gr.Interface.load("models/EleutherAI/gpt-neo-1.3B", description=description, examples=examples)
-            demo.launch()
         """
-        return super().load(name=name, src=src, api_key=api_key, alias=alias, **kwargs)
+        warnings.warn("gr.Interface.load() will be deprecated. Use gr.load() instead.")
+        return external.load(
+            name=name, src=src, hf_token=api_key, alias=alias, **kwargs
+        )
 
     @classmethod
     def from_pipeline(cls, pipeline: Pipeline, **kwargs) -> Interface:
@@ -133,7 +131,7 @@ class Interface(Blocks):
         description: str | None = None,
         article: str | None = None,
         thumbnail: str | None = None,
-        theme: Theme | None = None,
+        theme: Theme | str | None = None,
         css: str | None = None,
         allow_flagging: str | None = None,
         flagging_options: List[str] | List[Tuple[str, str]] | None = None,
@@ -242,10 +240,10 @@ class Interface(Blocks):
             self.cache_examples = False
 
         self.input_components = [
-            get_component_instance(i, render=False) for i in inputs
+            get_component_instance(i, render=False) for i in inputs  # type: ignore
         ]
         self.output_components = [
-            get_component_instance(o, render=False) for o in outputs
+            get_component_instance(o, render=False) for o in outputs  # type: ignore
         ]
 
         for component in self.input_components + self.output_components:
@@ -308,7 +306,6 @@ class Interface(Blocks):
         self.article = article
 
         self.thumbnail = thumbnail
-        self.theme = theme
 
         self.examples = examples
         self.num_shap = num_shap
@@ -352,9 +349,9 @@ class Interface(Blocks):
             raise ValueError(
                 "flagging_options must be a list of strings or list of (string, string) tuples."
             )
-        elif all([isinstance(x, str) for x in flagging_options]):
+        elif all(isinstance(x, str) for x in flagging_options):
             self.flagging_options = [(f"Flag as {x}", x) for x in flagging_options]
-        elif all([isinstance(x, tuple) for x in flagging_options]):
+        elif all(isinstance(x, tuple) for x in flagging_options):
             self.flagging_options = flagging_options
         else:
             raise ValueError(
@@ -372,22 +369,6 @@ class Interface(Blocks):
         self.local_url = None
 
         self.favicon_path = None
-
-        if self.analytics_enabled:
-            data = {
-                "mode": self.mode,
-                "fn": fn,
-                "inputs": inputs,
-                "outputs": outputs,
-                "live": live,
-                "interpretation": interpretation,
-                "allow_flagging": allow_flagging,
-                "custom_css": self.css is not None,
-                "theme": self.theme,
-                "version": GRADIO_VERSION,
-            }
-            utils.initiated_analytics(data)
-
         utils.version_check()
         Interface.instances.add(self)
 
@@ -404,7 +385,7 @@ class Interface(Blocks):
                 if len(self.output_components) == 1:
                     component.label = "output"
                 else:
-                    component.label = "output " + str(i)
+                    component.label = f"output {i}"
 
         if self.allow_flagging != "never":
             if (
@@ -480,9 +461,7 @@ class Interface(Blocks):
     def render_title_description(self) -> None:
         if self.title:
             Markdown(
-                "<h1 style='text-align: center; margin-bottom: 1rem'>"
-                + self.title
-                + "</h1>"
+                f"<h1 style='text-align: center; margin-bottom: 1rem'>{self.title}</h1>"
             )
         if self.description:
             Markdown(self.description)
@@ -641,7 +620,7 @@ class Interface(Blocks):
                         for output in self.fn(*args):
                             if len(self.output_components) == 1 and not self.batch:
                                 output = [output]
-                            output = [o for o in output]
+                            output = list(output)
                             yield output + [
                                 Button.update(visible=False),
                                 Button.update(visible=True),
@@ -804,13 +783,13 @@ class Interface(Blocks):
 
     def __repr__(self):
         repr = f"Gradio Interface for: {self.__name__}"
-        repr += "\n" + "-" * len(repr)
+        repr += f"\n{'-' * len(repr)}"
         repr += "\ninputs:"
         for component in self.input_components:
-            repr += "\n|-{}".format(str(component))
+            repr += f"\n|-{component}"
         repr += "\noutputs:"
         for component in self.output_components:
-            repr += "\n|-{}".format(str(component))
+            repr += f"\n|-{component}"
         return repr
 
     async def interpret_func(self, *args):
@@ -869,13 +848,11 @@ class TabbedInterface(Blocks):
             css=css,
         )
         if tab_names is None:
-            tab_names = ["Tab {}".format(i) for i in range(len(interface_list))]
+            tab_names = [f"Tab {i}" for i in range(len(interface_list))]
         with self:
             if title:
                 Markdown(
-                    "<h1 style='text-align: center; margin-bottom: 1rem'>"
-                    + title
-                    + "</h1>"
+                    f"<h1 style='text-align: center; margin-bottom: 1rem'>{title}</h1>"
                 )
             with Tabs():
                 for (interface, tab_name) in zip(interface_list, tab_names):
