@@ -9,15 +9,13 @@ import socket
 import threading
 import time
 import warnings
-from typing import TYPE_CHECKING, Optional, Tuple
+from typing import TYPE_CHECKING, Tuple
 
-import fastapi
 import requests
 import uvicorn
 
-from gradio import queueing
 from gradio.routes import App
-from gradio.tunneling import create_tunnel
+from gradio.tunneling import Tunnel
 
 if TYPE_CHECKING:  # Only import for type checking (to avoid circular imports).
     from gradio.blocks import Blocks
@@ -27,7 +25,7 @@ if TYPE_CHECKING:  # Only import for type checking (to avoid circular imports).
 INITIAL_PORT_VALUE = int(os.getenv("GRADIO_SERVER_PORT", "7860"))
 TRY_NUM_PORTS = int(os.getenv("GRADIO_NUM_PORTS", "100"))
 LOCALHOST_NAME = os.getenv("GRADIO_SERVER_NAME", "127.0.0.1")
-GRADIO_API_SERVER = "https://api.gradio.app/v1/tunnel-request"
+GRADIO_API_SERVER = "https://api.gradio.app/v2/tunnel-request"
 
 
 class Server(uvicorn.Server):
@@ -45,7 +43,7 @@ class Server(uvicorn.Server):
         self.thread.join()
 
 
-def get_first_available_port(initial: int, final: int, reuse_port: bool = False) -> int:
+def get_first_available_port(initial: int, final: int) -> int:
     """
     Gets the first open port in a specified range of port numbers
     Parameters:
@@ -57,21 +55,18 @@ def get_first_available_port(initial: int, final: int, reuse_port: bool = False)
     for port in range(initial, final):
         try:
             s = socket.socket()  # create a socket object
-            if reuse_port:
-                s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
             s.bind((LOCALHOST_NAME, port))  # Bind to the port
             s.close()
             return port
         except OSError:
             pass
     raise OSError(
-        "All ports from {} to {} are in use. Please close a port.".format(
-            initial, final - 1
-        )
+        f"All ports from {initial} to {final - 1} are in use. Please close a port."
     )
 
 
-def configure_app(app: fastapi.FastAPI, blocks: Blocks) -> fastapi.FastAPI:
+def configure_app(app: App, blocks: Blocks) -> App:
     auth = blocks.auth
     if auth is not None:
         if not callable(auth):
@@ -89,12 +84,12 @@ def configure_app(app: fastapi.FastAPI, blocks: Blocks) -> fastapi.FastAPI:
 
 def start_server(
     blocks: Blocks,
-    server_name: Optional[str] = None,
-    server_port: Optional[int] = None,
-    ssl_keyfile: Optional[str] = None,
-    ssl_certfile: Optional[str] = None,
-    ssl_keyfile_password: Optional[str] = None,
-) -> Tuple[int, str, fastapi.FastAPI, Server]:
+    server_name: str | None = None,
+    server_port: int | None = None,
+    ssl_keyfile: str | None = None,
+    ssl_certfile: str | None = None,
+    ssl_keyfile_password: str | None = None,
+) -> Tuple[str, int, str, App, Server]:
     """Launches a local server running the provided Interface
     Parameters:
     blocks: The Blocks object to run on the server
@@ -119,14 +114,14 @@ def start_server(
     else:
         try:
             s = socket.socket()
+            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
             s.bind((LOCALHOST_NAME, server_port))
             s.close()
-        except OSError:
+        except OSError as err:
             raise OSError(
-                "Port {} is in use. If a gradio.Blocks is running on the port, you can close() it or gradio.close_all().".format(
-                    server_port
-                )
-            )
+                f"Port {server_port} is in use. If a gradio.Blocks is running on the port, "
+                f"you can close() it or gradio.close_all()."
+            ) from err
         port = server_port
 
     url_host_name = "localhost" if server_name == "0.0.0.0" else server_name
@@ -136,58 +131,62 @@ def start_server(
             raise ValueError(
                 "ssl_certfile must be provided if ssl_keyfile is provided."
             )
-        path_to_local_server = "https://{}:{}/".format(url_host_name, port)
+        path_to_local_server = f"https://{url_host_name}:{port}/"
     else:
-        path_to_local_server = "http://{}:{}/".format(url_host_name, port)
+        path_to_local_server = f"http://{url_host_name}:{port}/"
+
+    # Strip IPv6 brackets from the address if they exist.
+    # This is needed as http://[::1]:port/ is a valid browser address,
+    # but not a valid IPv6 address, so asyncio will throw an exception.
+    if server_name.startswith("[") and server_name.endswith("]"):
+        host = server_name[1:-1]
+    else:
+        host = server_name
 
     app = App.create_app(blocks)
 
-    if app.blocks.enable_queue:
-        if blocks.auth is not None or app.blocks.encrypt:
-            raise ValueError("Cannot queue with encryption or authentication enabled.")
-        queueing.init()
-        app.queue_thread = threading.Thread(
-            target=queueing.queue_thread, args=(path_to_local_server,)
-        )
-        app.queue_thread.start()
     if blocks.save_to is not None:  # Used for selenium tests
         blocks.save_to["port"] = port
     config = uvicorn.Config(
         app=app,
         port=port,
-        host=server_name,
+        host=host,
         log_level="warning",
         ssl_keyfile=ssl_keyfile,
         ssl_certfile=ssl_certfile,
         ssl_keyfile_password=ssl_keyfile_password,
+        ws_max_size=1024 * 1024 * 1024,  # Setting max websocket size to be 1 GB
     )
     server = Server(config=config)
     server.run_in_thread()
-    return port, path_to_local_server, app, server
+    return server_name, port, path_to_local_server, app, server
 
 
-def setup_tunnel(local_server_port: int, endpoint: str) -> str:
-    response = requests.get(
-        endpoint + "/v1/tunnel-request" if endpoint is not None else GRADIO_API_SERVER
-    )
+def setup_tunnel(local_host: str, local_port: int, share_token: str) -> str:
+    response = requests.get(GRADIO_API_SERVER)
     if response and response.status_code == 200:
         try:
             payload = response.json()[0]
-            return create_tunnel(payload, LOCALHOST_NAME, local_server_port)
+            remote_host, remote_port = payload["host"], int(payload["port"])
+            tunnel = Tunnel(
+                remote_host, remote_port, local_host, local_port, share_token
+            )
+            address = tunnel.start_tunnel()
+            return address
         except Exception as e:
-            raise RuntimeError(str(e))
-    else:
-        raise RuntimeError("Could not get share link from Gradio API Server.")
+            raise RuntimeError(str(e)) from e
+    raise RuntimeError("Could not get share link from Gradio API Server.")
 
 
 def url_ok(url: str) -> bool:
     try:
         for _ in range(5):
-            time.sleep(0.500)
             with warnings.catch_warnings():
                 warnings.filterwarnings("ignore")
                 r = requests.head(url, timeout=3, verify=False)
             if r.status_code in (200, 401, 302):  # 401 or 302 if auth is set
                 return True
+            time.sleep(0.500)
     except (ConnectionError, requests.exceptions.ConnectionError):
         return False
+    return False
