@@ -67,9 +67,9 @@ class Queue:
         self.access_token = ""
         self.queue_client = None
 
-    async def start(self, progress_tracking=False):
+    async def start(self, progress_tracking=False, ssl_verify=True):
         # So that the client is attached to the running event loop
-        self.queue_client = httpx.AsyncClient()
+        self.queue_client = httpx.AsyncClient(verify=ssl_verify)
 
         run_coro_in_background(self.start_processing)
         if progress_tracking:
@@ -122,7 +122,7 @@ class Queue:
                 await asyncio.sleep(self.sleep_when_free)
                 continue
 
-            if not (None in self.active_jobs):
+            if None not in self.active_jobs:
                 await asyncio.sleep(self.sleep_when_free)
                 continue
             # Using mutex to avoid editing a list in use
@@ -209,9 +209,9 @@ class Queue:
     async def gather_event_data(self, event: Event, receive_timeout=60) -> bool:
         """
         Gather data for the event
-
         Parameters:
-            event:
+            event: the Event to gather data for
+            receive_timeout: how long to wait for data to be received from frontend
         """
         if not event.data:
             client_awake = await self.send_message(event, {"msg": "send_data"})
@@ -309,7 +309,7 @@ class Queue:
             "headers": dict(websocket.headers),
             "query_params": dict(websocket.query_params),
             "path_params": dict(websocket.path_params),
-            "client": dict(host=websocket.client.host, port=websocket.client.port),  # type: ignore
+            "client": {"host": websocket.client.host, "port": websocket.client.port},  # type: ignore
         }
 
     async def call_prediction(self, events: List[Event], batch: bool):
@@ -430,7 +430,7 @@ class Queue:
                 except Exception:
                     pass
             self.active_jobs[self.active_jobs.index(events)] = None
-            for event in awake_events:
+            for event in events:
                 await self.clean_event(event)
                 # Always reset the state of the iterator
                 # If the job finished successfully, this has no effect
@@ -444,7 +444,7 @@ class Queue:
                 event.websocket.send_json(data=data), timeout=timeout
             )
             return True
-        except:
+        except Exception:
             await self.clean_event(event)
             return False
 
