@@ -8,13 +8,13 @@ import inspect
 import json
 import mimetypes
 import os
-import posixpath
 import secrets
 import tempfile
 import traceback
 from asyncio import TimeoutError as AsyncTimeOutError
 from collections import defaultdict
 from copy import deepcopy
+from pathlib import Path
 from typing import Any, Dict, List, Optional, Type
 from urllib.parse import urlparse
 
@@ -33,6 +33,7 @@ from fastapi.responses import (
 )
 from fastapi.security import OAuth2PasswordRequestForm
 from fastapi.templating import Jinja2Templates
+from gradio_client.documentation import document, set_documentation_group
 from jinja2.exceptions import TemplateNotFound
 from starlette.background import BackgroundTask
 from starlette.responses import RedirectResponse, StreamingResponse
@@ -43,10 +44,8 @@ import gradio.ranged_response as ranged_response
 from gradio import utils
 from gradio.context import Context
 from gradio.data_classes import PredictBody, ResetBody
-from gradio.documentation import document, set_documentation_group
 from gradio.exceptions import Error
 from gradio.helpers import EventData
-from gradio.processing_utils import TempFileManager
 from gradio.queueing import Estimation, Event
 from gradio.utils import cancel_tasks, run_coro_in_background, set_task_name
 
@@ -113,8 +112,10 @@ class App(FastAPI):
         self.lock = asyncio.Lock()
         self.queue_token = secrets.token_urlsafe(32)
         self.startup_events_triggered = False
-        self.uploaded_file_dir = str(utils.abspath(tempfile.mkdtemp()))
-        super().__init__(**kwargs)
+        self.uploaded_file_dir = os.environ.get("GRADIO_TEMP_DIR") or str(
+            Path(tempfile.gettempdir()) / "gradio"
+        )
+        super().__init__(**kwargs, docs_url=None, redoc_url=None)
 
     def configure_app(self, blocks: gradio.Blocks) -> None:
         auth = blocks.auth
@@ -161,14 +162,16 @@ class App(FastAPI):
         @app.get("/login_check")
         @app.get("/login_check/")
         def login_check(user: str = Depends(get_current_user)):
-            if app.auth is None or not (user is None):
+            if app.auth is None or user is not None:
                 return
             raise HTTPException(
                 status_code=status.HTTP_401_UNAUTHORIZED, detail="Not authenticated"
             )
 
         async def ws_login_check(websocket: WebSocket) -> Optional[str]:
-            token = websocket.cookies.get("access-token")
+            token = websocket.cookies.get("access-token") or websocket.cookies.get(
+                "access-token-unsecure"
+            )
             return token  # token is returned to allow request in queue
 
         @app.get("/token")
@@ -220,13 +223,14 @@ class App(FastAPI):
             mimetypes.add_type("application/javascript", ".js")
             blocks = app.get_blocks()
 
-            if app.auth is None or not (user is None):
+            if app.auth is None or user is not None:
                 config = app.get_blocks().config
             else:
                 config = {
                     "auth_required": True,
                     "auth_message": blocks.auth_message,
                     "is_space": app.get_blocks().is_space,
+                    "root": app.get_blocks().root,
                 }
 
             try:
@@ -237,17 +241,23 @@ class App(FastAPI):
                     template,
                     {"request": request, "config": config},
                 )
-            except TemplateNotFound:
+            except TemplateNotFound as err:
                 if blocks.share:
                     raise ValueError(
                         "Did you install Gradio from source files? Share mode only "
                         "works when Gradio is installed through the pip package."
-                    )
+                    ) from err
                 else:
                     raise ValueError(
                         "Did you install Gradio from source files? You need to build "
                         "the frontend by running /scripts/build_frontend.sh"
-                    )
+                    ) from err
+
+        @app.get("/info/", dependencies=[Depends(login_check)])
+        @app.get("/info", dependencies=[Depends(login_check)])
+        def api_info(serialize: bool = True):
+            config = app.get_blocks().config
+            return gradio.blocks.get_api_info(config, serialize)  # type: ignore
 
         @app.get("/config/", dependencies=[Depends(login_check)])
         @app.get("/config", dependencies=[Depends(login_check)])
@@ -257,16 +267,12 @@ class App(FastAPI):
         @app.get("/static/{path:path}")
         def static_resource(path: str):
             static_file = safe_join(STATIC_PATH_LIB, path)
-            if static_file is not None:
-                return FileResponse(static_file)
-            raise HTTPException(status_code=404, detail="Static file not found")
+            return FileResponse(static_file)
 
         @app.get("/assets/{path:path}")
         def build_resource(path: str):
             build_file = safe_join(BUILD_PATH_LIB, path)
-            if build_file is not None:
-                return FileResponse(build_file)
-            raise HTTPException(status_code=404, detail="Build file not found")
+            return FileResponse(build_file)
 
         @app.get("/favicon.ico")
         async def favicon():
@@ -282,8 +288,8 @@ class App(FastAPI):
             # Adapted from: https://github.com/tiangolo/fastapi/issues/1788
             url = httpx.URL(url_path)
             headers = {}
-            if Context.access_token is not None:
-                headers["Authorization"] = f"Bearer {Context.access_token}"
+            if Context.hf_token is not None:
+                headers["Authorization"] = f"Bearer {Context.hf_token}"
             rp_req = client.build_request("GET", url, headers=headers)
             rp_resp = await client.send(rp_req, stream=True)
             return StreamingResponse(
@@ -301,21 +307,32 @@ class App(FastAPI):
                 return RedirectResponse(
                     url=path_or_url, status_code=status.HTTP_302_FOUND
                 )
-            abs_path = str(utils.abspath(path_or_url))
-            in_app_dir = utils.abspath(app.cwd) in utils.abspath(path_or_url).parents
-            created_by_app = abs_path in set().union(*blocks.temp_file_sets)
-            in_file_dir = any(
+            abs_path = utils.abspath(path_or_url)
+            in_blocklist = any(
                 (
-                    utils.abspath(dir) in utils.abspath(path_or_url).parents
-                    for dir in blocks.file_directories
+                    utils.is_in_or_equal(abs_path, blocked_path)
+                    for blocked_path in blocks.blocked_paths
                 )
             )
-            was_uploaded = (
-                utils.abspath(app.uploaded_file_dir)
-                in utils.abspath(path_or_url).parents
+            if in_blocklist:
+                raise HTTPException(403, f"File not allowed: {path_or_url}.")
+
+            in_app_dir = utils.abspath(app.cwd) in abs_path.parents
+            created_by_app = str(abs_path) in set().union(*blocks.temp_file_sets)
+            in_file_dir = any(
+                (
+                    utils.is_in_or_equal(abs_path, allowed_path)
+                    for allowed_path in blocks.allowed_paths
+                )
             )
+            was_uploaded = utils.abspath(app.uploaded_file_dir) in abs_path.parents
 
             if in_app_dir or created_by_app or in_file_dir or was_uploaded:
+                if not abs_path.exists():
+                    raise HTTPException(404, "File not found")
+                if abs_path.is_dir():
+                    raise HTTPException(403)
+
                 range_val = request.headers.get("Range", "").strip()
                 if range_val.startswith("bytes=") and "-" in range_val:
                     range_val = range_val[6:]
@@ -333,8 +350,9 @@ class App(FastAPI):
                 return FileResponse(abs_path, headers={"Accept-Ranges": "bytes"})
 
             else:
-                raise ValueError(
-                    f"File cannot be fetched: {path_or_url}. All files must contained within the Gradio python app working directory, or be a temp file created by the Gradio python app."
+                raise HTTPException(
+                    403,
+                    f"File cannot be fetched: {path_or_url}. All files must contained within the Gradio python app working directory, or be a temp file created by the Gradio python app.",
                 )
 
         @app.get("/file/{path:path}", dependencies=[Depends(login_check)])
@@ -372,7 +390,7 @@ class App(FastAPI):
                 # the job being cancelled will not overwrite the state of the iterator.
                 # In all cases, should_reset will be the empty set the next time
                 # the fn_index is run.
-                app.iterators[body.session_hash]["should_reset"] = set([])
+                app.iterators[body.session_hash]["should_reset"] = set()
             else:
                 session_state = {}
                 iterators = {}
@@ -390,15 +408,16 @@ class App(FastAPI):
             if not (body.batched) and batch:
                 raw_input = [raw_input]
             try:
-                output = await app.get_blocks().process_api(
-                    fn_index=fn_index_inferred,
-                    inputs=raw_input,
-                    request=request,
-                    state=session_state,
-                    iterators=iterators,
-                    event_id=event_id,
-                    event_data=event_data,
-                )
+                with utils.MatplotlibBackendMananger():
+                    output = await app.get_blocks().process_api(
+                        fn_index=fn_index_inferred,
+                        inputs=raw_input,
+                        request=request,
+                        state=session_state,
+                        iterators=iterators,
+                        event_id=event_id,
+                        event_data=event_data,
+                    )
                 iterator = output.pop("iterator", None)
                 if hasattr(body, "session_hash"):
                     if fn_index in app.iterators[body.session_hash]["should_reset"]:
@@ -514,7 +533,7 @@ class App(FastAPI):
             # Continuous events are not put in the queue  so that they do not
             # occupy the queue's resource as they are expected to run forever
             if blocks.dependencies[event.fn_index].get("every", 0):
-                await cancel_tasks(set([f"{event.session_hash}_{event.fn_index}"]))
+                await cancel_tasks({f"{event.session_hash}_{event.fn_index}"})
                 await blocks._queue.reset_iterators(event.session_hash, event.fn_index)
                 task = run_coro_in_background(
                     blocks._queue.process_events, [event], False
@@ -547,7 +566,7 @@ class App(FastAPI):
             files: List[UploadFile] = File(...),
         ):
             output_files = []
-            file_manager = TempFileManager()
+            file_manager = gradio.File()
             for input_file in files:
                 output_files.append(
                     await file_manager.save_uploaded_file(
@@ -584,26 +603,31 @@ class App(FastAPI):
 ########
 
 
-def safe_join(directory: str, path: str) -> str | None:
+def safe_join(directory: str, path: str) -> str:
     """Safely path to a base directory to avoid escaping the base directory.
     Borrowed from: werkzeug.security.safe_join"""
-    _os_alt_seps: List[str] = list(
+    _os_alt_seps: List[str] = [
         sep for sep in [os.path.sep, os.path.altsep] if sep is not None and sep != "/"
-    )
+    ]
 
-    if path != "":
-        filename = posixpath.normpath(path)
-    else:
-        return directory
+    if path == "":
+        raise HTTPException(400)
 
+    filename = os.path.normpath(path)
+    fullpath = os.path.join(directory, filename)
     if (
         any(sep in filename for sep in _os_alt_seps)
         or os.path.isabs(filename)
         or filename == ".."
         or filename.startswith("../")
+        or os.path.isdir(fullpath)
     ):
-        return None
-    return posixpath.join(directory, filename)
+        raise HTTPException(403)
+
+    if not os.path.exists(fullpath):
+        raise HTTPException(404, "File not found")
+
+    return fullpath
 
 
 def get_types(cls_set: List[Type]):
@@ -724,8 +748,10 @@ class Request:
         else:
             try:
                 obj = self.kwargs[name]
-            except KeyError:
-                raise AttributeError(f"'Request' object has no attribute '{name}'")
+            except KeyError as ke:
+                raise AttributeError(
+                    f"'Request' object has no attribute '{name}'"
+                ) from ke
             return self.dict_to_obj(obj)
 
 
@@ -757,6 +783,7 @@ def mount_gradio_app(
     blocks.dev_mode = False
     blocks.root = path[:-1] if path.endswith("/") else path
     blocks.config = blocks.get_config_file()
+    blocks.validate_queue_settings()
     gradio_app = App.create_app(blocks)
 
     @app.on_event("startup")
