@@ -7,4 +7,4 @@ function make_script(src) {
     script.src = src;
     document.head.appendChild(script);
 }
-make_script("https://gradio.s3-us-west-2.amazonaws.com/3.22.1/assets/index.d6979a87.js");
+make_script("https://gradio.s3-us-west-2.amazonaws.com/3.28.2/assets/index-a7a243d7.js");
