@@ -1,14 +1,34 @@
 from __future__ import annotations
 
+import json
 import re
-from typing import Iterable
+import tempfile
+import textwrap
+from pathlib import Path
+from typing import Dict, Iterable
 
-from gradio.themes.utils import colors, fonts, sizes
+import huggingface_hub
+import requests
+import semantic_version as semver
+from gradio_client.documentation import document, set_documentation_group
+from huggingface_hub import CommitOperationAdd
+
+from gradio.themes.utils import (
+    colors,
+    fonts,
+    get_matching_version,
+    get_theme_assets,
+    sizes,
+)
+from gradio.themes.utils.readme_content import README_CONTENT
+
+set_documentation_group("themes")
 
 
 class ThemeClass:
     def __init__(self):
         self._stylesheets = []
+        self.name = None
 
     def _get_theme_css(self):
         css = {}
@@ -72,9 +92,237 @@ class ThemeClass:
             + "\n}"
         )
 
-        return css_code + "\n" + dark_css_code
+        return f"{css_code}\n{dark_css_code}"
+
+    def to_dict(self):
+        """Convert the theme into a python dictionary."""
+        schema = {"theme": {}}
+        for prop in dir(self):
+            if (
+                not prop.startswith("_")
+                or prop.startswith("_font")
+                or prop == "_stylesheets"
+                or prop == "name"
+            ) and isinstance(getattr(self, prop), (list, str)):
+                schema["theme"][prop] = getattr(self, prop)
+        return schema
+
+    @classmethod
+    def load(cls, path: str) -> "ThemeClass":
+        """Load a theme from a json file.
+
+        Parameters:
+            path: The filepath to read.
+        """
+        theme = json.load(open(path), object_hook=fonts.as_font)
+        return cls.from_dict(theme)
+
+    @classmethod
+    def from_dict(cls, theme: Dict[str, Dict[str, str]]) -> "ThemeClass":
+        """Create a theme instance from a dictionary representation.
+
+        Parameters:
+            theme: The dictionary representation of the theme.
+        """
+        new_theme = cls()
+        for prop, value in theme["theme"].items():
+            setattr(new_theme, prop, value)
+
+        # For backwards compatibility, load attributes in base theme not in the loaded theme from the base theme.
+        base = Base()
+        for attr in base.__dict__:
+            if not attr.startswith("_") and not hasattr(new_theme, attr):
+                setattr(new_theme, attr, getattr(base, attr))
+
+        return new_theme
+
+    def dump(self, filename: str):
+        """Write the theme to a json file.
+
+        Parameters:
+            filename: The path to write the theme too
+        """
+        as_dict = self.to_dict()
+        json.dump(as_dict, open(Path(filename), "w"), cls=fonts.FontEncoder)
+
+    @classmethod
+    def from_hub(cls, repo_name: str, hf_token: str | None = None):
+        """Load a theme from the hub.
+
+        This DOES NOT require a HuggingFace account for downloading publicly available themes.
+
+        Parameters:
+            repo_name: string of the form <author>/<theme-name>@<semantic-version-expression>.  If a semantic version expression is omitted, the latest version will be fetched.
+            hf_token: HuggingFace Token. Only needed to download private themes.
+        """
+        if "@" not in repo_name:
+            name, version = repo_name, None
+        else:
+            name, version = repo_name.split("@")
+
+        api = huggingface_hub.HfApi(token=hf_token)
+
+        try:
+            space_info = api.space_info(name)
+        except requests.HTTPError as e:
+            raise ValueError(f"The space {name} does not exist") from e
+
+        assets = get_theme_assets(space_info)
+        matching_version = get_matching_version(assets, version)
+
+        if not matching_version:
+            raise ValueError(
+                f"Cannot find a matching version for expression {version} "
+                f"from files {[f.filename for f in assets]}"
+            )
+
+        theme_file = huggingface_hub.hf_hub_download(
+            repo_id=name,
+            repo_type="space",
+            filename=f"themes/theme_schema@{matching_version.version}.json",
+        )
+        theme = cls.load(theme_file)
+        theme.name = name
+        return theme
+
+    @staticmethod
+    def _get_next_version(space_info: huggingface_hub.hf_api.SpaceInfo) -> str:
+        assets = get_theme_assets(space_info)
+        latest_version = max(assets, key=lambda asset: asset.version).version
+        return str(latest_version.next_patch())
+
+    @staticmethod
+    def _theme_version_exists(
+        space_info: huggingface_hub.hf_api.SpaceInfo, version: str
+    ) -> bool:
+        assets = get_theme_assets(space_info)
+        return any(a.version == semver.Version(version) for a in assets)
+
+    def push_to_hub(
+        self,
+        repo_name: str,
+        org_name: str | None = None,
+        version: str | None = None,
+        hf_token: str | None = None,
+        theme_name: str | None = None,
+        description: str | None = None,
+        private: bool = False,
+    ):
+        """Upload a theme to the HuggingFace hub.
+
+        This requires a HuggingFace account.
+
+        Parameters:
+            repo_name: The name of the repository to store the theme assets, e.g. 'my_theme' or 'sunset'.
+            org_name: The name of the org to save the space in. If None (the default), the username corresponding to the logged in user, or hƒ_token is used.
+            version: A semantic version tag for theme. Bumping the version tag lets you publish updates to a theme without changing the look of applications that already loaded your theme.
+            hf_token: API token for your HuggingFace account
+            theme_name: Name for the name. If None, defaults to repo_name
+            description: A long form description to your theme.
+        """
+
+        from gradio import __version__
+
+        api = huggingface_hub.HfApi()
+
+        if not hf_token:
+            try:
+                author = huggingface_hub.whoami()["name"]
+            except OSError as e:
+                raise ValueError(
+                    "In order to push to hub, log in via `huggingface-cli login` "
+                    "or provide a theme_token to push_to_hub. For more information "
+                    "see https://huggingface.co/docs/huggingface_hub/quick-start#login"
+                ) from e
+        else:
+            author = huggingface_hub.whoami(token=hf_token)["name"]
+
+        space_id = f"{org_name or author}/{repo_name}"
+
+        try:
+            space_info = api.space_info(space_id)
+        except requests.HTTPError:
+            space_info = None
+
+        space_exists = space_info is not None
+
+        # If no version, set the version to next patch release
+        if not version:
+            if space_exists:
+                version = self._get_next_version(space_info)
+            else:
+                version = "0.0.1"
+        else:
+            _ = semver.Version(version)
+
+        if space_exists and self._theme_version_exists(space_info, version):
+            raise ValueError(
+                f"The space {space_id} already has a "
+                f"theme with version {version}. See: themes/theme_schema@{version}.json. "
+                "To manually override this version, use the HuggingFace hub UI."
+            )
+
+        theme_name = theme_name or repo_name
+
+        with tempfile.NamedTemporaryFile(
+            mode="w", delete=False, suffix=".json"
+        ) as css_file:
+            contents = self.to_dict()
+            contents["version"] = version
+            json.dump(contents, css_file, cls=fonts.FontEncoder)
+        with tempfile.NamedTemporaryFile(mode="w", delete=False) as readme_file:
+            readme_content = README_CONTENT.format(
+                theme_name=theme_name,
+                description=description or "Add a description of this theme here!",
+                author=author,
+                gradio_version=__version__,
+            )
+            readme_file.write(textwrap.dedent(readme_content))
+        with tempfile.NamedTemporaryFile(mode="w", delete=False) as app_file:
+            contents = open(str(Path(__file__).parent / "app.py")).read()
+            contents = re.sub(
+                r"theme=gr.themes.Default\(\)",
+                f"theme='{space_id}'",
+                contents,
+            )
+            contents = re.sub(r"{THEME}", theme_name or repo_name, contents)
+            contents = re.sub(r"{AUTHOR}", org_name or author, contents)
+            contents = re.sub(r"{SPACE_NAME}", repo_name, contents)
+            app_file.write(contents)
+
+        operations = [
+            CommitOperationAdd(
+                path_in_repo=f"themes/theme_schema@{version}.json",
+                path_or_fileobj=css_file.name,
+            ),
+            CommitOperationAdd(
+                path_in_repo="README.md", path_or_fileobj=readme_file.name
+            ),
+            CommitOperationAdd(path_in_repo="app.py", path_or_fileobj=app_file.name),
+        ]
+
+        huggingface_hub.create_repo(
+            space_id,
+            repo_type="space",
+            space_sdk="gradio",
+            token=hf_token,
+            exist_ok=True,
+            private=private,
+        )
+
+        api.create_commit(
+            repo_id=space_id,
+            commit_message="Updating theme",
+            repo_type="space",
+            operations=operations,
+            token=hf_token,
+        )
+        url = f"https://huggingface.co/spaces/{space_id}"
+        print(f"See your theme here! {url}")
+        return url
 
 
+@document("push_to_hub", "from_hub", "load", "dump", "from_dict", "to_dict")
 class Base(ThemeClass):
     def __init__(
         self,
@@ -114,6 +362,8 @@ class Base(ThemeClass):
             font_mono: The monospace font to use for the theme, applies to code. Pass a string for a system font, or a gradio.themes.font.GoogleFont object to load a font from Google Fonts. Pass a list of fonts for fallbacks.
         """
 
+        self.name = "base"
+
         def expand_shortcut(shortcut, mode="color", prefix=None):
             if not isinstance(shortcut, str):
                 return shortcut
@@ -124,7 +374,7 @@ class Base(ThemeClass):
                 raise ValueError(f"Color shortcut {shortcut} not found.")
             elif mode == "size":
                 for size in sizes.Size.all:
-                    if size.name == prefix + "_" + shortcut:
+                    if size.name == f"{prefix}_{shortcut}":
                         return size
                 raise ValueError(f"Size shortcut {shortcut} not found.")
 
@@ -224,7 +474,7 @@ class Base(ThemeClass):
     def set(
         self,
         *,
-        # Body
+        # Body Attributes: These set set the values for the entire body of the app.
         body_background_fill=None,
         body_background_fill_dark=None,
         body_text_color=None,
@@ -234,7 +484,7 @@ class Base(ThemeClass):
         body_text_color_subdued_dark=None,
         body_text_weight=None,
         embed_radius=None,
-        # Core Colors
+        # Element Colors: These set the colors for common elements.
         background_fill_primary=None,
         background_fill_primary_dark=None,
         background_fill_secondary=None,
@@ -246,7 +496,7 @@ class Base(ThemeClass):
         color_accent=None,
         color_accent_soft=None,
         color_accent_soft_dark=None,
-        # Text
+        # Text: This sets the text styling for text elements.
         link_text_color=None,
         link_text_color_dark=None,
         link_text_color_active=None,
@@ -258,13 +508,13 @@ class Base(ThemeClass):
         prose_text_size=None,
         prose_text_weight=None,
         prose_header_text_weight=None,
-        # Shadows
+        # Shadows: These set the high-level shadow rendering styles. These variables are often referenced by other component-specific shadow variables.
         shadow_drop=None,
         shadow_drop_lg=None,
         shadow_inset=None,
         shadow_spread=None,
         shadow_spread_dark=None,
-        # Layout Atoms
+        # Layout Atoms: These set the style for common layout elements, such as the blocks that wrap components.
         block_background_fill=None,
         block_background_fill_dark=None,
         block_border_color=None,
@@ -281,6 +531,7 @@ class Base(ThemeClass):
         block_label_border_color_dark=None,
         block_label_border_width=None,
         block_label_border_width_dark=None,
+        block_label_shadow=None,
         block_label_text_color=None,
         block_label_text_color_dark=None,
         block_label_margin=None,
@@ -316,7 +567,9 @@ class Base(ThemeClass):
         panel_border_width_dark=None,
         section_header_text_size=None,
         section_header_text_weight=None,
-        # Component Atoms
+        # Component Atoms: These set the style for elements within components.
+        chatbot_code_background_color=None,
+        chatbot_code_background_color_dark=None,
         checkbox_background_color=None,
         checkbox_background_color_dark=None,
         checkbox_background_color_focus=None,
@@ -407,25 +660,21 @@ class Base(ThemeClass):
         table_radius=None,
         table_row_focus=None,
         table_row_focus_dark=None,
-        # Buttons
+        # Buttons: These set the style for buttons.
         button_border_width=None,
         button_border_width_dark=None,
-        button_cancel_background_fill=None,
-        button_cancel_background_fill_dark=None,
-        button_cancel_background_fill_hover=None,
-        button_cancel_background_fill_hover_dark=None,
-        button_cancel_border_color=None,
-        button_cancel_border_color_dark=None,
-        button_cancel_border_color_hover=None,
-        button_cancel_border_color_hover_dark=None,
-        button_cancel_text_color=None,
-        button_cancel_text_color_dark=None,
-        button_cancel_text_color_hover=None,
-        button_cancel_text_color_hover_dark=None,
+        button_shadow=None,
+        button_shadow_active=None,
+        button_shadow_hover=None,
+        button_transition=None,
         button_large_padding=None,
         button_large_radius=None,
         button_large_text_size=None,
         button_large_text_weight=None,
+        button_small_padding=None,
+        button_small_radius=None,
+        button_small_text_size=None,
+        button_small_text_weight=None,
         button_primary_background_fill=None,
         button_primary_background_fill_dark=None,
         button_primary_background_fill_hover=None,
@@ -450,14 +699,18 @@ class Base(ThemeClass):
         button_secondary_text_color_dark=None,
         button_secondary_text_color_hover=None,
         button_secondary_text_color_hover_dark=None,
-        button_shadow=None,
-        button_shadow_active=None,
-        button_shadow_hover=None,
-        button_small_padding=None,
-        button_small_radius=None,
-        button_small_text_size=None,
-        button_small_text_weight=None,
-        button_transition=None,
+        button_cancel_background_fill=None,
+        button_cancel_background_fill_dark=None,
+        button_cancel_background_fill_hover=None,
+        button_cancel_background_fill_hover_dark=None,
+        button_cancel_border_color=None,
+        button_cancel_border_color_dark=None,
+        button_cancel_border_color_hover=None,
+        button_cancel_border_color_hover_dark=None,
+        button_cancel_text_color=None,
+        button_cancel_text_color_dark=None,
+        button_cancel_text_color_hover=None,
+        button_cancel_text_color_hover_dark=None,
     ) -> Base:
         """
         Parameters:
@@ -513,6 +766,7 @@ class Base(ThemeClass):
             block_label_border_color_dark: The border color of the title label of a media element (e.g. image) in dark mode.
             block_label_border_width: The border width of the title label of a media element (e.g. image).
             block_label_border_width_dark: The border width of the title label of a media element (e.g. image) in dark mode.
+            block_label_shadow: The shadow of the title label of a media element (e.g. image).
             block_label_text_color: The text color of the title label of a media element (e.g. image).
             block_label_text_color_dark: The text color of the title label of a media element (e.g. image) in dark mode.
             block_label_margin: The margin of the title label of a media element (e.g. image) from its surrounding container.
@@ -548,6 +802,8 @@ class Base(ThemeClass):
             panel_border_width_dark: The border width of a panel in dark mode.
             section_header_text_size: The text size of a section header (e.g. tab name).
             section_header_text_weight: The text weight of a section header (e.g. tab name).
+            chatbot_code_background_color: The background color of code blocks in the chatbot.
+            chatbot_code_background_color_dark: The background color of code blocks in the chatbot in dark mode.
             checkbox_background_color: The background of a checkbox square or radio circle.
             checkbox_background_color_dark: The background of a checkbox square or radio circle in dark mode.
             checkbox_background_color_focus: The background of a checkbox square or radio circle when focused.
@@ -843,6 +1099,9 @@ class Base(ThemeClass):
         self.block_label_border_width_dark = block_label_border_width_dark or getattr(
             self, "block_label_border_width_dark", None
         )
+        self.block_label_shadow = block_label_shadow or getattr(
+            self, "block_label_shadow", "*block_shadow"
+        )
         self.block_label_text_color = block_label_text_color or getattr(
             self, "block_label_text_color", "*neutral_500"
         )
@@ -946,6 +1205,13 @@ class Base(ThemeClass):
             self, "section_header_text_weight", "400"
         )
         # Component Atoms
+        self.chatbot_code_background_color = chatbot_code_background_color or getattr(
+            self, "chatbot_code_background_color", "*neutral_100"
+        )
+        self.chatbot_code_background_color_dark = (
+            chatbot_code_background_color_dark
+            or getattr(self, "chatbot_code_background_color_dark", "*neutral_800")
+        )
         self.checkbox_background_color = checkbox_background_color or getattr(
             self, "checkbox_background_color", "*background_fill_primary"
         )
@@ -1260,16 +1526,15 @@ class Base(ThemeClass):
         self.prose_header_text_weight = prose_header_text_weight or getattr(
             self, "prose_header_text_weight", "600"
         )
-        self.slider_color = slider_color or getattr(self, "slider_color", "")
+        self.slider_color = slider_color or getattr(self, "slider_color", "auto")
         self.slider_color_dark = slider_color_dark or getattr(
             self, "slider_color_dark", None
         )
         self.stat_background_fill = stat_background_fill or getattr(
             self, "stat_background_fill", "*primary_300"
         )
-        self.stat_background_fill_dark = (
-            stat_background_fill_dark
-            or getattr(self, "stat_background_fill_dark", "*primary_500")
+        self.stat_background_fill_dark = stat_background_fill_dark or getattr(
+            self, "stat_background_fill_dark", "*primary_500"
         )
         self.table_border_color = table_border_color or getattr(
             self, "table_border_color", "*neutral_300"
