@@ -4,6 +4,7 @@ from __future__ import annotations
 
 import asyncio
 import copy
+import functools
 import inspect
 import json
 import json.decoder
@@ -29,7 +30,6 @@ from typing import (
     Dict,
     Generator,
     List,
-    NewType,
     Tuple,
     Type,
     TypeVar,
@@ -37,22 +37,23 @@ from typing import (
 )
 
 import aiohttp
-import fsspec.asyn
 import httpx
-import matplotlib.pyplot as plt
+import matplotlib
 import requests
-from huggingface_hub.utils import send_telemetry
 from markdown_it import MarkdownIt
+from pygments import highlight
+from pygments.formatters import HtmlFormatter
+from pygments.lexers import get_lexer_by_name
 from mdit_py_plugins.dollarmath.index import dollarmath_plugin
 from mdit_py_plugins.footnote.index import footnote_plugin
-from pydantic import BaseModel, Json, parse_obj_as
+from pydantic import BaseModel, parse_obj_as
 
 import gradio
 from gradio.context import Context
 from gradio.strings import en
 
 if TYPE_CHECKING:  # Only import for type checking (is False at runtime).
-    from gradio.blocks import BlockContext
+    from gradio.blocks import Block, BlockContext
     from gradio.components import Component
 
 analytics_url = "https://api.gradio.app/"
@@ -76,18 +77,15 @@ def version_check():
         ]
         if StrictVersion(latest_pkg_version) > StrictVersion(current_pkg_version):
             print(
-                "IMPORTANT: You are using gradio version {}, "
-                "however version {} "
-                "is available, please upgrade.".format(
-                    current_pkg_version, latest_pkg_version
-                )
+                f"IMPORTANT: You are using gradio version {current_pkg_version}, "
+                f"however version {latest_pkg_version} is available, please upgrade."
             )
             print("--------")
     except json.decoder.JSONDecodeError:
         warnings.warn("unable to parse version details from package URL.")
     except KeyError:
         warnings.warn("package URL does not contain version info.")
-    except:
+    except Exception:
         pass
 
 
@@ -112,24 +110,12 @@ def initiated_analytics(data: Dict[str, Any]) -> None:
     def initiated_analytics_thread(data: Dict[str, Any]) -> None:
         try:
             requests.post(
-                analytics_url + "gradio-initiated-analytics/", data=data, timeout=3
+                f"{analytics_url}gradio-initiated-analytics/", data=data, timeout=5
             )
         except (requests.ConnectionError, requests.exceptions.ReadTimeout):
             pass  # do not push analytics if no network
 
-    def initiated_telemetry_thread(data: Dict[str, Any]) -> None:
-        try:
-            send_telemetry(
-                topic="gradio/initiated",
-                library_name="gradio",
-                library_version=GRADIO_VERSION,
-                user_agent=data,
-            )
-        except Exception:
-            pass
-
     threading.Thread(target=initiated_analytics_thread, args=(data,)).start()
-    threading.Thread(target=initiated_telemetry_thread, args=(data,)).start()
 
 
 def launch_analytics(data: Dict[str, Any]) -> None:
@@ -138,7 +124,7 @@ def launch_analytics(data: Dict[str, Any]) -> None:
     def launch_analytics_thread(data: Dict[str, Any]) -> None:
         try:
             requests.post(
-                analytics_url + "gradio-launched-analytics/", data=data, timeout=3
+                f"{analytics_url}gradio-launched-analytics/", data=data, timeout=5
             )
         except (requests.ConnectionError, requests.exceptions.ReadTimeout):
             pass  # do not push analytics if no network
@@ -172,13 +158,13 @@ def launched_telemetry(blocks: gradio.Blocks, data: Dict[str, Any]) -> None:
             str(blocks.blocks[y]) for y in x["outputs"]
         ]
     additional_data = {
+        "version": GRADIO_VERSION,
         "is_kaggle": blocks.is_kaggle,
         "is_sagemaker": blocks.is_sagemaker,
         "using_auth": blocks.auth is not None,
         "dev_mode": blocks.dev_mode,
         "show_api": blocks.show_api,
         "show_error": blocks.show_error,
-        "theme": blocks.theme,
         "title": blocks.title,
         "inputs": blocks.input_components
         if blocks.mode == "interface"
@@ -192,17 +178,15 @@ def launched_telemetry(blocks: gradio.Blocks, data: Dict[str, Any]) -> None:
     }
 
     data.update(additional_data)
+    data.update({"ip_address": get_local_ip_address()})
 
     def launched_telemtry_thread(data: Dict[str, Any]) -> None:
         try:
-            send_telemetry(
-                topic="gradio/launched",
-                library_name="gradio",
-                library_version=GRADIO_VERSION,
-                user_agent=data,
+            requests.post(
+                f"{analytics_url}gradio-launched-telemetry/", data=data, timeout=5
             )
-        except Exception as e:
-            print("Error while sending telemetry: {}".format(e))
+        except Exception:
+            pass
 
     threading.Thread(target=launched_telemtry_thread, args=(data,)).start()
 
@@ -213,55 +197,31 @@ def integration_analytics(data: Dict[str, Any]) -> None:
     def integration_analytics_thread(data: Dict[str, Any]) -> None:
         try:
             requests.post(
-                analytics_url + "gradio-integration-analytics/", data=data, timeout=3
+                f"{analytics_url}gradio-integration-analytics/", data=data, timeout=5
             )
         except (requests.ConnectionError, requests.exceptions.ReadTimeout):
             pass  # do not push analytics if no network
 
-    def integration_telemetry_thread(data: Dict[str, Any]) -> None:
-        try:
-            send_telemetry(
-                topic="gradio/integration",
-                library_name="gradio",
-                library_version=GRADIO_VERSION,
-                user_agent=data,
-            )
-        except Exception as e:
-            print("Error while sending telemetry: {}".format(e))
-
     threading.Thread(target=integration_analytics_thread, args=(data,)).start()
-    threading.Thread(target=integration_telemetry_thread, args=(data,)).start()
 
 
 def error_analytics(message: str) -> None:
     """
     Send error analytics if there is network
-    :param ip_address: IP address where error occurred
-    :param message: Details about error
+    Parameters:
+        message: Details about error
     """
     data = {"ip_address": get_local_ip_address(), "error": message}
 
     def error_analytics_thread(data: Dict[str, Any]) -> None:
         try:
             requests.post(
-                analytics_url + "gradio-error-analytics/", data=data, timeout=3
+                f"{analytics_url}gradio-error-analytics/", data=data, timeout=5
             )
         except (requests.ConnectionError, requests.exceptions.ReadTimeout):
             pass  # do not push analytics if no network
 
-    def error_telemetry_thread(data: Dict[str, Any]) -> None:
-        try:
-            send_telemetry(
-                topic="gradio/error",
-                library_name="gradio",
-                library_version=GRADIO_VERSION,
-                user_agent=message,
-            )
-        except Exception as e:
-            print("Error while sending telemetry: {}".format(e))
-
     threading.Thread(target=error_analytics_thread, args=(data,)).start()
-    threading.Thread(target=error_telemetry_thread, args=(data,)).start()
 
 
 async def log_feature_analytics(feature: str) -> None:
@@ -269,7 +229,7 @@ async def log_feature_analytics(feature: str) -> None:
     async with aiohttp.ClientSession() as session:
         try:
             async with session.post(
-                analytics_url + "gradio-feature-analytics/", data=data
+                f"{analytics_url}gradio-feature-analytics/", data=data
             ):
                 pass
         except (aiohttp.ClientError):
@@ -306,7 +266,7 @@ def sagemaker_check() -> bool:
         client = boto3.client("sts")
         response = client.get_caller_identity()
         return "sagemaker" in response["Arn"].lower()
-    except:
+    except Exception:
         return False
 
 
@@ -356,7 +316,7 @@ def launch_counter() -> None:
                 print(en["BETA_INVITE"])
             with open(JSON_PATH, "w") as j:
                 j.write(json.dumps(launches))
-    except:
+    except Exception:
         pass
 
 
@@ -498,24 +458,6 @@ def component_or_layout_class(cls_name: str) -> Type[Component] | Type[BlockCont
     raise ValueError(f"No such component or layout: {cls_name}")
 
 
-def synchronize_async(func: Callable, *args, **kwargs) -> Any:
-    """
-    Runs async functions in sync scopes.
-
-    Can be used in any scope. See run_coro_in_background for more details.
-
-    Example:
-        if inspect.iscoroutinefunction(block_fn.fn):
-            predictions = utils.synchronize_async(block_fn.fn, *processed_input)
-
-    Args:
-        func:
-        *args:
-        **kwargs:
-    """
-    return fsspec.asyn.sync(fsspec.asyn.get_loop(), func, *args, **kwargs)
-
-
 def run_coro_in_background(func: Callable, *args, **kwargs):
     """
     Runs coroutines in background.
@@ -549,7 +491,7 @@ def async_iteration(iterator):
         return next(iterator)
     except StopIteration:
         # raise a ValueError here because co-routines can't raise StopIteration themselves
-        raise StopAsyncIteration()
+        raise StopAsyncIteration() from None
 
 
 class AsyncRequest:
@@ -571,7 +513,6 @@ class AsyncRequest:
     You can see example usages in test_utils.py.
     """
 
-    ResponseJson = NewType("ResponseJson", Json)
     client = httpx.AsyncClient()
 
     class Method(str, Enum):
@@ -674,9 +615,7 @@ class AsyncRequest:
         request = httpx.Request(method, url, **kwargs)
         return request
 
-    def _validate_response_data(
-        self, response: ResponseJson
-    ) -> Union[BaseModel, ResponseJson | None]:
+    def _validate_response_data(self, response):
         """
         Validate response using given validation methods. If there is a validation method and response is not valid,
         validation functions will raise an exception for them.
@@ -705,7 +644,7 @@ class AsyncRequest:
 
         return validated_response
 
-    def _validate_response_by_model(self, response: ResponseJson) -> BaseModel:
+    def _validate_response_by_model(self, response) -> BaseModel:
         """
         Validate response json using the validation model.
         Args:
@@ -718,9 +657,7 @@ class AsyncRequest:
             validated_data = parse_obj_as(self._validation_model, response)
         return validated_data
 
-    def _validate_response_by_validation_function(
-        self, response: ResponseJson
-    ) -> ResponseJson | None:
+    def _validate_response_by_validation_function(self, response):
         """
         Validate response json using the validation function.
         Args:
@@ -787,19 +724,6 @@ def set_directory(path: Path | str):
         os.chdir(origin)
 
 
-def strip_invalid_filename_characters(filename: str, max_bytes: int = 200) -> str:
-    """Strips invalid characters from a filename and ensures that the file_length is less than `max_bytes` bytes."""
-    filename = "".join([char for char in filename if char.isalnum() or char in "._- "])
-    filename_len = len(filename.encode())
-    if filename_len > max_bytes:
-        while filename_len > max_bytes:
-            if len(filename) == 0:
-                break
-            filename = filename[:-1]
-            filename_len = len(filename.encode())
-    return filename
-
-
 def sanitize_value_for_csv(value: str | Number) -> str | Number:
     """
     Sanitizes a value that is being written to a CSV file to prevent CSV injection attacks.
@@ -812,7 +736,7 @@ def sanitize_value_for_csv(value: str | Number) -> str | Number:
     if any(value.startswith(prefix) for prefix in unsafe_prefixes) or any(
         sequence in value for sequence in unsafe_sequences
     ):
-        value = "'" + value
+        value = f"'{value}"
     return value
 
 
@@ -839,10 +763,10 @@ def append_unique_suffix(name: str, list_of_names: List[str]):
         return name
     else:
         suffix_counter = 1
-        new_name = name + f"_{suffix_counter}"
+        new_name = f"{name}_{suffix_counter}"
         while new_name in set_of_names:
             suffix_counter += 1
-            new_name = name + f"_{suffix_counter}"
+            new_name = f"{name}_{suffix_counter}"
         return new_name
 
 
@@ -904,7 +828,7 @@ def get_cancel_function(
             ]
 
     async def cancel(session_hash: str) -> None:
-        task_ids = set([f"{session_hash}_{fn}" for fn in fn_to_comp])
+        task_ids = {f"{session_hash}_{fn}" for fn in fn_to_comp}
         await cancel_tasks(task_ids)
 
     return (
@@ -913,6 +837,14 @@ def get_cancel_function(
     )
 
 
+def get_type_hints(fn):
+    if inspect.isfunction(fn) or inspect.ismethod(fn):
+        return typing.get_type_hints(fn)
+    elif callable(fn):
+        return typing.get_type_hints(fn.__call__)
+    return {}
+
+
 def check_function_inputs_match(fn: Callable, inputs: List, inputs_as_dict: bool):
     """
     Checks if the input component set matches the function
@@ -930,7 +862,7 @@ def check_function_inputs_match(fn: Callable, inputs: List, inputs_as_dict: bool
         return is_request or is_event_data
 
     signature = inspect.signature(fn)
-    parameter_types = typing.get_type_hints(fn) if inspect.isfunction(fn) else {}
+    parameter_types = get_type_hints(fn)
     min_args = 0
     max_args = 0
     infinity = -1
@@ -970,43 +902,122 @@ class TupleNoPrint(tuple):
         return ""
 
 
+class MatplotlibBackendMananger:
+    def __enter__(self):
+        self._original_backend = matplotlib.get_backend()
+        matplotlib.use("agg")
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        matplotlib.use(self._original_backend)
+
+
 def tex2svg(formula, *args):
-    FONTSIZE = 20
-    DPI = 300
-    plt.rc("mathtext", fontset="cm")
-    fig = plt.figure(figsize=(0.01, 0.01))
-    fig.text(0, 0, r"${}$".format(formula), fontsize=FONTSIZE)
-    output = BytesIO()
-    fig.savefig(
-        output,
-        dpi=DPI,
-        transparent=True,
-        format="svg",
-        bbox_inches="tight",
-        pad_inches=0.0,
-    )
-    plt.close(fig)
-    output.seek(0)
-    xml_code = output.read().decode("utf-8")
-    svg_start = xml_code.index("<svg ")
-    svg_code = xml_code[svg_start:]
-    svg_code = re.sub(r"<metadata>.*<\/metadata>", "", svg_code, flags=re.DOTALL)
-    svg_code = re.sub(r' width="[^"]+"', "", svg_code)
-    height_match = re.search(r'height="([\d.]+)pt"', svg_code)
-    if height_match:
-        height = float(height_match.group(1))
-        new_height = height / FONTSIZE  # conversion from pt to em
-        svg_code = re.sub(r'height="[\d.]+pt"', f'height="{new_height}em"', svg_code)
-    copy_code = f"<span style='font-size: 0px'>{formula}</span>"
+    with MatplotlibBackendMananger():
+        import matplotlib.pyplot as plt
+
+        FONTSIZE = 20
+        DPI = 300
+        plt.rc("mathtext", fontset="cm")
+        fig = plt.figure(figsize=(0.01, 0.01))
+        fig.text(0, 0, rf"${formula}$", fontsize=FONTSIZE)
+        output = BytesIO()
+        fig.savefig(
+            output,
+            dpi=DPI,
+            transparent=True,
+            format="svg",
+            bbox_inches="tight",
+            pad_inches=0.0,
+        )
+        plt.close(fig)
+        output.seek(0)
+        xml_code = output.read().decode("utf-8")
+        svg_start = xml_code.index("<svg ")
+        svg_code = xml_code[svg_start:]
+        svg_code = re.sub(r"<metadata>.*<\/metadata>", "", svg_code, flags=re.DOTALL)
+        svg_code = re.sub(r' width="[^"]+"', "", svg_code)
+        height_match = re.search(r'height="([\d.]+)pt"', svg_code)
+        if height_match:
+            height = float(height_match.group(1))
+            new_height = height / FONTSIZE  # conversion from pt to em
+            svg_code = re.sub(
+                r'height="[\d.]+pt"', f'height="{new_height}em"', svg_code
+            )
+        copy_code = f"<span style='font-size: 0px'>{formula}</span>"
     return f"{copy_code}{svg_code}"
 
 
 def abspath(path: str | Path) -> Path:
     """Returns absolute path of a str or Path path, but does not resolve symlinks."""
-    if Path(path).is_symlink():
+    path = Path(path)
+
+    if path.is_absolute():
+        return path
+
+    # recursively check if there is a symlink within the path
+    is_symlink = path.is_symlink() or any(
+        parent.is_symlink() for parent in path.parents
+    )
+
+    if is_symlink or path == path.resolve():  # in case path couldn't be resolved
         return Path.cwd() / path
     else:
-        return Path(path).resolve()
+        return path.resolve()
+
+
+def is_in_or_equal(path_1: str | Path, path_2: str | Path):
+    """
+    True if path_1 is a descendant (i.e. located within) path_2 or if the paths are the
+    same, returns False otherwise.
+    Parameters:
+        path_1: str or Path (can be a file or directory)
+        path_2: str or Path (can be a file or directory)
+    """
+    return (abspath(path_2) in abspath(path_1).parents) or abspath(path_1) == abspath(
+        path_2
+    )
+
+
+def get_serializer_name(block: Block) -> str | None:
+    if not hasattr(block, "serialize"):
+        return None
+
+    def get_class_that_defined_method(meth: Callable):
+        # Adapted from: https://stackoverflow.com/a/25959545/5209347
+        if isinstance(meth, functools.partial):
+            return get_class_that_defined_method(meth.func)
+        if inspect.ismethod(meth) or (
+            inspect.isbuiltin(meth)
+            and getattr(meth, "__self__", None) is not None
+            and getattr(meth.__self__, "__class__", None)
+        ):
+            for cls in inspect.getmro(meth.__self__.__class__):
+                if meth.__name__ in cls.__dict__:
+                    return cls
+            meth = getattr(meth, "__func__", meth)  # fallback to __qualname__ parsing
+        if inspect.isfunction(meth):
+            cls = getattr(
+                inspect.getmodule(meth),
+                meth.__qualname__.split(".<locals>", 1)[0].rsplit(".", 1)[0],
+                None,
+            )
+            if isinstance(cls, type):
+                return cls
+        return getattr(meth, "__objclass__", None)
+
+    cls = get_class_that_defined_method(block.serialize)  # type: ignore
+    if cls:
+        return cls.__name__
+
+
+def highlight_code(code, name, attrs):
+    try:
+        lexer = get_lexer_by_name(name)
+    except:
+        lexer = get_lexer_by_name("text")
+    formatter = HtmlFormatter()
+
+    return highlight(code, lexer, formatter)
 
 
 def get_markdown_parser() -> MarkdownIt:
@@ -1017,7 +1028,7 @@ def get_markdown_parser() -> MarkdownIt:
                 "linkify": True,
                 "typographer": True,
                 "html": True,
-                "breaks": True,
+                "highlight": highlight_code,
             },
         )
         .use(dollarmath_plugin, renderer=tex2svg, allow_digits=False)
