@@ -15,20 +15,19 @@ from difflib import SequenceMatcher
 from pathlib import Path
 from unittest.mock import MagicMock, patch
 
-import matplotlib
-import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import PIL
 import pytest
 import vega_datasets
+from gradio_client import media_data
+from gradio_client import utils as client_utils
 from scipy.io import wavfile
 
 import gradio as gr
-from gradio import media_data, processing_utils
+from gradio import processing_utils, utils
 
 os.environ["GRADIO_ANALYTICS_ENABLED"] = "False"
-matplotlib.use("Agg")
 
 
 class TestComponent:
@@ -50,7 +49,7 @@ def test_raise_warnings():
 class TestTextbox:
     def test_component_functions(self):
         """
-        Preprocess, postprocess, serialize, tokenize, generate_sample, get_config
+        Preprocess, postprocess, serialize, tokenize, get_config
         """
         text_input = gr.Textbox()
         assert text_input.preprocess("Hello World!") == "Hello World!"
@@ -93,11 +92,11 @@ class TestTextbox:
             "label": None,
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
         }
-        assert isinstance(text_input.generate_sample(), str)
 
     @pytest.mark.asyncio
     async def test_in_interface_as_input(self):
@@ -182,10 +181,10 @@ class TestTextbox:
 class TestNumber:
     def test_component_functions(self):
         """
-        Preprocess, postprocess, serialize, generate_sample, set_interpret_parameters, get_interpretation_neighbors, get_config
+        Preprocess, postprocess, serialize, set_interpret_parameters, get_interpretation_neighbors, get_config
 
         """
-        numeric_input = gr.Number()
+        numeric_input = gr.Number(elem_id="num", elem_classes="first")
         assert numeric_input.preprocess(3) == 3.0
         assert numeric_input.preprocess(None) is None
         assert numeric_input.postprocess(3) == 3
@@ -193,7 +192,6 @@ class TestNumber:
         assert numeric_input.postprocess(2.14) == 2.14
         assert numeric_input.postprocess(None) is None
         assert numeric_input.serialize(3, True) == 3
-        assert isinstance(numeric_input.generate_sample(), float)
         numeric_input.set_interpret_parameters(steps=3, delta=1, delta_type="absolute")
         assert numeric_input.get_interpretation_neighbors(1) == (
             [-2.0, -1.0, 0.0, 2.0, 3.0, 4.0],
@@ -210,7 +208,8 @@ class TestNumber:
             "show_label": True,
             "label": None,
             "style": {},
-            "elem_id": None,
+            "elem_id": "num",
+            "elem_classes": ["first"],
             "visible": True,
             "interactive": None,
             "root_url": None,
@@ -218,7 +217,7 @@ class TestNumber:
 
     def test_component_functions_integer(self):
         """
-        Preprocess, postprocess, serialize, generate_sample, set_interpret_parameters, get_interpretation_neighbors, get_template_context
+        Preprocess, postprocess, serialize, set_interpret_parameters, get_interpretation_neighbors, get_template_context
 
         """
         numeric_input = gr.Number(precision=0, value=42)
@@ -229,7 +228,6 @@ class TestNumber:
         assert numeric_input.postprocess(2.85) == 3
         assert numeric_input.postprocess(None) is None
         assert numeric_input.serialize(3, True) == 3
-        assert isinstance(numeric_input.generate_sample(), int)
         numeric_input.set_interpret_parameters(steps=3, delta=1, delta_type="absolute")
         assert numeric_input.get_interpretation_neighbors(1) == (
             [-2.0, -1.0, 0.0, 2.0, 3.0, 4.0],
@@ -256,6 +254,7 @@ class TestNumber:
             "label": None,
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
@@ -263,7 +262,7 @@ class TestNumber:
 
     def test_component_functions_precision(self):
         """
-        Preprocess, postprocess, serialize, generate_sample, set_interpret_parameters, get_interpretation_neighbors, get_template_context
+        Preprocess, postprocess, serialize, set_interpret_parameters, get_interpretation_neighbors, get_template_context
 
         """
         numeric_input = gr.Number(precision=2, value=42.3428)
@@ -351,7 +350,7 @@ class TestNumber:
 class TestSlider:
     def test_component_functions(self):
         """
-        Preprocess, postprocess, serialize, generate_sample, get_config
+        Preprocess, postprocess, serialize, get_config
         """
         slider_input = gr.Slider()
         assert slider_input.preprocess(3.0) == 3.0
@@ -360,7 +359,6 @@ class TestSlider:
         assert slider_input.postprocess(None) == 0
         assert slider_input.serialize(3, True) == 3
 
-        assert isinstance(slider_input.generate_sample(), int)
         slider_input = gr.Slider(10, 20, value=15, step=1, label="Slide Your Input")
         assert slider_input.get_config() == {
             "minimum": 10,
@@ -372,6 +370,7 @@ class TestSlider:
             "label": "Slide Your Input",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
@@ -427,14 +426,13 @@ class TestSlider:
 class TestCheckbox:
     def test_component_functions(self):
         """
-        Preprocess, postprocess, serialize, generate_sample, get_config
+        Preprocess, postprocess, serialize, get_config
         """
         bool_input = gr.Checkbox()
         assert bool_input.preprocess(True)
         assert bool_input.postprocess(True)
         assert bool_input.postprocess(True)
         assert bool_input.serialize(True, True)
-        assert isinstance(bool_input.generate_sample(), bool)
         bool_input = gr.Checkbox(value=True, label="Check Your Input")
         assert bool_input.get_config() == {
             "value": True,
@@ -443,6 +441,7 @@ class TestCheckbox:
             "label": "Check Your Input",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
@@ -467,13 +466,12 @@ class TestCheckbox:
 class TestCheckboxGroup:
     def test_component_functions(self):
         """
-        Preprocess, postprocess, serialize, generate_sample, get_config
+        Preprocess, postprocess, serialize, get_config
         """
         checkboxes_input = gr.CheckboxGroup(["a", "b", "c"])
         assert checkboxes_input.preprocess(["a", "c"]) == ["a", "c"]
         assert checkboxes_input.postprocess(["a", "c"]) == ["a", "c"]
         assert checkboxes_input.serialize(["a", "c"], True) == ["a", "c"]
-        assert isinstance(checkboxes_input.generate_sample(), list)
         checkboxes_input = gr.CheckboxGroup(
             value=["a", "c"],
             choices=["a", "b", "c"],
@@ -487,6 +485,7 @@ class TestCheckboxGroup:
             "label": "Check Your Inputs",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
@@ -512,14 +511,13 @@ class TestCheckboxGroup:
 class TestRadio:
     def test_component_functions(self):
         """
-        Preprocess, postprocess, serialize, generate_sample, get_config
+        Preprocess, postprocess, serialize, get_config
 
         """
         radio_input = gr.Radio(["a", "b", "c"])
         assert radio_input.preprocess("c") == "c"
         assert radio_input.postprocess("a") == "a"
         assert radio_input.serialize("a", True) == "a"
-        assert isinstance(radio_input.generate_sample(), str)
         radio_input = gr.Radio(
             choices=["a", "b", "c"], default="a", label="Pick Your One Input"
         )
@@ -531,6 +529,7 @@ class TestRadio:
             "label": "Pick Your One Input",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
@@ -558,7 +557,7 @@ class TestRadio:
 class TestDropdown:
     def test_component_functions(self):
         """
-        Preprocess, postprocess, serialize, generate_sample, get_config
+        Preprocess, postprocess, serialize, get_config
         """
         dropdown_input = gr.Dropdown(["a", "b", "c"], multiselect=True)
         assert dropdown_input.preprocess("a") == "a"
@@ -568,7 +567,6 @@ class TestDropdown:
         assert dropdown_input_multiselect.preprocess(["a", "c"]) == ["a", "c"]
         assert dropdown_input_multiselect.postprocess(["a", "c"]) == ["a", "c"]
         assert dropdown_input_multiselect.serialize(["a", "c"], True) == ["a", "c"]
-        assert isinstance(dropdown_input_multiselect.generate_sample(), str)
         dropdown_input_multiselect = gr.Dropdown(
             value=["a", "c"],
             choices=["a", "b", "c"],
@@ -577,6 +575,7 @@ class TestDropdown:
             max_choices=2,
         )
         assert dropdown_input_multiselect.get_config() == {
+            "allow_custom_value": False,
             "choices": ["a", "b", "c"],
             "value": ["a", "c"],
             "name": "dropdown",
@@ -584,6 +583,7 @@ class TestDropdown:
             "label": "Select Your Inputs",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
@@ -611,7 +611,7 @@ class TestDropdown:
 class TestImage:
     def test_component_functions(self):
         """
-        Preprocess, postprocess, serialize, generate_sample, get_config, _segment_by_slic
+        Preprocess, postprocess, serialize, get_config, _segment_by_slic
         type: pil, file, filepath, numpy
         """
         img = deepcopy(media_data.BASE64_IMAGE)
@@ -623,8 +623,10 @@ class TestImage:
         assert image_input.preprocess(img).size == (30, 10)
         assert image_input.postprocess("test/test_files/bus.png") == img
         assert image_input.serialize("test/test_files/bus.png") == img
+        image_input = gr.Image(type="filepath")
+        image_temp_filepath = image_input.preprocess(img)
+        assert image_temp_filepath in image_input.temp_files
 
-        assert isinstance(image_input.generate_sample(), str)
         image_input = gr.Image(
             source="upload", tool="editor", type="pil", label="Upload Your Image"
         )
@@ -640,11 +642,13 @@ class TestImage:
             "label": "Upload Your Image",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "value": None,
             "interactive": None,
             "root_url": None,
             "mirror_webcam": True,
+            "selectable": False,
         }
         assert image_input.preprocess(None) is None
         image_input = gr.Image(invert_colors=True)
@@ -676,7 +680,7 @@ class TestImage:
     @pytest.mark.flaky
     def test_serialize_url(self):
         img = "https://gradio.app/assets/img/header-image.jpg"
-        expected = processing_utils.encode_url_or_file_to_base64(img)
+        expected = client_utils.encode_url_or_file_to_base64(img)
         assert gr.Image().serialize(img) == expected
 
     def test_in_interface_as_input(self):
@@ -727,12 +731,15 @@ class TestPlot:
         """
 
         def plot(num):
+            import matplotlib.pyplot as plt
+
             fig = plt.figure()
             plt.plot(range(num), range(num))
             return fig
 
         iface = gr.Interface(plot, "slider", "plot")
-        output = await iface.process_api(fn_index=0, inputs=[10], state={})
+        with utils.MatplotlibBackendMananger():
+            output = await iface.process_api(fn_index=0, inputs=[10], state={})
         assert output["data"][0]["type"] == "matplotlib"
         assert output["data"][0]["plot"].startswith("data:image/png;base64")
 
@@ -740,8 +747,11 @@ class TestPlot:
         """
         postprocess
         """
-        fig = plt.figure()
-        plt.plot([1, 2, 3], [1, 2, 3])
+        with utils.MatplotlibBackendMananger():
+            import matplotlib.pyplot as plt
+
+            fig = plt.figure()
+            plt.plot([1, 2, 3], [1, 2, 3])
 
         component = gr.Plot(fig)
         assert component.get_config().get("value") is not None
@@ -770,7 +780,7 @@ class TestPlot:
 class TestAudio:
     def test_component_functions(self):
         """
-        Preprocess, postprocess serialize, generate_sample, get_config, deserialize
+        Preprocess, postprocess serialize, get_config, deserialize
         type: filepath, numpy, file
         """
         x_wav = deepcopy(media_data.BASE64_AUDIO)
@@ -789,7 +799,6 @@ class TestAudio:
             audio_input.serialize("test/test_files/audio_sample.wav")["name"],
         )
 
-        assert isinstance(audio_input.generate_sample(), dict)
         audio_input = gr.Audio(label="Upload Your Audio")
         assert audio_input.get_config() == {
             "source": "upload",
@@ -799,6 +808,7 @@ class TestAudio:
             "label": "Upload Your Audio",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "value": None,
             "interactive": None,
@@ -817,7 +827,7 @@ class TestAudio:
             gr.Audio(type="unknown")
 
         # Output functionalities
-        y_audio = gr.processing_utils.decode_base64_to_file(
+        y_audio = client_utils.decode_base64_to_file(
             deepcopy(media_data.BASE64_AUDIO)["data"]
         )
         audio_output = gr.Audio(type="filepath")
@@ -830,6 +840,7 @@ class TestAudio:
             "source": "upload",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "value": None,
             "interactive": None,
@@ -875,7 +886,7 @@ class TestAudio:
         iface = gr.Interface(reverse_audio, "audio", "audio")
         reversed_file = iface("test/test_files/audio_sample.wav")
         reversed_reversed_file = iface(reversed_file)
-        reversed_reversed_data = gr.processing_utils.encode_url_or_file_to_base64(
+        reversed_reversed_data = client_utils.encode_url_or_file_to_base64(
             reversed_reversed_file
         )
         similarity = SequenceMatcher(
@@ -904,7 +915,7 @@ class TestAudio:
 class TestFile:
     def test_component_functions(self):
         """
-        Preprocess, serialize, generate_sample, get_config, value
+        Preprocess, serialize, get_config, value
         """
         x_file = deepcopy(media_data.BASE64_FILE)
         file_input = gr.File()
@@ -924,7 +935,6 @@ class TestFile:
         assert input1.name == input2.name
         assert Path(input1.name).name == "sample_file.pdf"
 
-        assert isinstance(file_input.generate_sample(), dict)
         file_input = gr.File(label="Upload Your File")
         assert file_input.get_config() == {
             "file_count": "single",
@@ -934,6 +944,7 @@ class TestFile:
             "label": "Upload Your File",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "value": None,
             "interactive": None,
@@ -1013,7 +1024,7 @@ class TestUploadButton:
 class TestDataframe:
     def test_component_functions(self):
         """
-        Preprocess, serialize, generate_sample, get_config
+        Preprocess, serialize, get_config
         """
         x_data = {
             "data": [["Tim", 12, False], ["Jan", 24, True]],
@@ -1025,7 +1036,6 @@ class TestDataframe:
         assert not output["Member"][0]
         assert dataframe_input.postprocess(x_data) == x_data
 
-        assert isinstance(dataframe_input.generate_sample(), list)
         dataframe_input = gr.Dataframe(
             headers=["Name", "Age", "Member"], label="Dataframe Input"
         )
@@ -1048,6 +1058,7 @@ class TestDataframe:
             "overflow_row_behaviour": "paginate",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
@@ -1070,6 +1081,7 @@ class TestDataframe:
             "label": None,
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "datatype": ["str", "str", "str"],
             "row_count": (1, "dynamic"),
@@ -1090,6 +1102,8 @@ class TestDataframe:
         postprocess
         """
         dataframe_output = gr.Dataframe()
+        output = dataframe_output.postprocess([])
+        assert output == {"data": [[]], "headers": []}
         output = dataframe_output.postprocess(np.zeros((2, 2)))
         assert output == {"data": [[0, 0], [0, 0]], "headers": [1, 2]}
         output = dataframe_output.postprocess([[1, 3, 5]])
@@ -1240,7 +1254,7 @@ class TestDataset:
 class TestVideo:
     def test_component_functions(self):
         """
-        Preprocess, serialize, deserialize, generate_sample, get_config
+        Preprocess, serialize, deserialize, get_config
         """
         x_video = deepcopy(media_data.BASE64_VIDEO)
         video_input = gr.Video()
@@ -1249,7 +1263,6 @@ class TestVideo:
         output2 = video_input.preprocess(x_video)
         assert output1 == output2
 
-        assert isinstance(video_input.generate_sample(), dict)
         video_input = gr.Video(label="Upload Your Video")
         assert video_input.get_config() == {
             "source": "upload",
@@ -1258,6 +1271,7 @@ class TestVideo:
             "label": "Upload Your Video",
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "value": None,
             "interactive": None,
@@ -1274,23 +1288,32 @@ class TestVideo:
         assert "flip" not in output_video
 
         assert filecmp.cmp(
-            video_input.serialize(x_video["name"])["name"], x_video["name"]
+            video_input.serialize(x_video["name"])[0]["name"], x_video["name"]
         )
 
         # Output functionalities
         y_vid_path = "test/test_files/video_sample.mp4"
+        subtitles_path = "test/test_files/s1.srt"
         video_output = gr.Video()
-        output1 = video_output.postprocess(y_vid_path)["name"]
+        output1 = video_output.postprocess(y_vid_path)[0]["name"]
         assert output1.endswith("mp4")
-        output2 = video_output.postprocess(y_vid_path)["name"]
+        output2 = video_output.postprocess(y_vid_path)[0]["name"]
         assert output1 == output2
+        assert (
+            video_output.postprocess(y_vid_path)[0]["orig_name"] == "video_sample.mp4"
+        )
+        output_with_subtitles = video_output.postprocess((y_vid_path, subtitles_path))
+        assert output_with_subtitles[1]["data"].startswith("data")
 
         assert video_output.deserialize(
-            {
-                "name": None,
-                "data": deepcopy(media_data.BASE64_VIDEO)["data"],
-                "is_file": False,
-            }
+            (
+                {
+                    "name": None,
+                    "data": deepcopy(media_data.BASE64_VIDEO)["data"],
+                    "is_file": False,
+                },
+                None,
+            )
         ).endswith(".mp4")
 
     def test_in_interface(self):
@@ -1383,7 +1406,7 @@ class TestVideo:
 class TestTimeseries:
     def test_component_functions(self):
         """
-        Preprocess, postprocess,  generate_sample, get_config,
+        Preprocess, postprocess,  get_config,
         """
         timeseries_input = gr.Timeseries(x="time", y=["retail", "food", "other"])
         x_timeseries = {
@@ -1393,7 +1416,6 @@ class TestTimeseries:
         output = timeseries_input.preprocess(x_timeseries)
         assert isinstance(output, pd.core.frame.DataFrame)
 
-        assert isinstance(timeseries_input.generate_sample(), dict)
         timeseries_input = gr.Timeseries(
             x="time", y="retail", label="Upload Your Timeseries"
         )
@@ -1406,6 +1428,7 @@ class TestTimeseries:
             "colors": None,
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "value": None,
             "interactive": None,
@@ -1428,6 +1451,7 @@ class TestTimeseries:
             "colors": None,
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "value": None,
             "interactive": None,
@@ -1452,7 +1476,7 @@ class TestNames:
     # This test ensures that `components.get_component_instance()` works correctly when instantiating from components
     def test_no_duplicate_uncased_names(self):
         subclasses = gr.components.Component.__subclasses__()
-        unique_subclasses_uncased = set([s.__name__.lower() for s in subclasses])
+        unique_subclasses_uncased = {s.__name__.lower() for s in subclasses}
         assert len(subclasses) == len(unique_subclasses_uncased)
 
 
@@ -1503,6 +1527,7 @@ class TestLabel:
             "label": None,
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
@@ -1642,6 +1667,7 @@ class TestHighlightedText:
             "show_legend": False,
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "value": None,
             "interactive": None,
@@ -1681,55 +1707,133 @@ class TestHighlightedText:
             ]
 
 
+class TestAnnotatedImage:
+    def test_postprocess(self):
+        """
+        postprocess
+        """
+        component = gr.AnnotatedImage()
+        img = np.random.randint(0, 255, (100, 100, 3), dtype=np.uint8)
+        mask1 = [40, 40, 50, 50]
+        mask2 = np.zeros((100, 100), dtype=np.uint8)
+        mask2[10:20, 10:20] = 1
+
+        input = (img, [(mask1, "mask1"), (mask2, "mask2")])
+        result = component.postprocess(input)
+
+        base_img_out, (mask1_out, mask2_out) = result
+        base_img_out = PIL.Image.open(base_img_out["name"])
+
+        assert mask1_out[1] == "mask1"
+
+        mask1_img_out = PIL.Image.open(mask1_out[0]["name"])
+        assert mask1_img_out.size == base_img_out.size
+        mask1_array_out = np.array(mask1_img_out)
+        assert np.max(mask1_array_out[40:50, 40:50]) == 255
+        assert np.max(mask1_array_out[50:60, 50:60]) == 0
+
+    def test_component_functions(self):
+        ht_output = gr.AnnotatedImage(label="sections", show_legend=False)
+        assert ht_output.get_config() == {
+            "name": "annotatedimage",
+            "show_label": True,
+            "label": "sections",
+            "show_legend": False,
+            "style": {},
+            "elem_id": None,
+            "elem_classes": None,
+            "visible": True,
+            "value": None,
+            "root_url": None,
+            "selectable": False,
+            "interactive": None,
+        }
+
+    def test_in_interface(self):
+        def mask(img):
+            top_left_corner = [0, 0, 20, 20]
+            random_mask = np.random.randint(0, 2, img.shape[:2])
+            return (img, [(top_left_corner, "left corner"), (random_mask, "random")])
+
+        iface = gr.Interface(mask, "image", gr.AnnotatedImage())
+        output_json = iface("test/test_files/bus.png")
+        with open(output_json) as fp:
+            output = json.load(fp)
+            output_img, (mask1, mask1) = output
+        input_img = PIL.Image.open("test/test_files/bus.png")
+        output_img = PIL.Image.open(output_img["name"])
+        mask1_img = PIL.Image.open(mask1[0]["name"])
+
+        assert output_img.size == input_img.size
+        assert mask1_img.size == input_img.size
+
+
 class TestChatbot:
     def test_component_functions(self):
         """
         Postprocess, get_config
         """
         chatbot = gr.Chatbot()
-        assert chatbot.postprocess([("You are **cool**", "so are *you*")]) == [
-            ("You are <strong>cool</strong>", "so are <em>you</em>")
+        assert chatbot.postprocess([["You are **cool**\nand fun", "so are *you*"]]) == [
+            ["You are <strong>cool</strong>\nand fun", "so are <em>you</em>"]
         ]
 
         multimodal_msg = [
-            (("driving.mp4",), "cool video"),
-            (("cantina.wav",), "cool audio"),
-            (("lion.jpg", "A lion"), "cool pic"),
+            [("test/test_files/video_sample.mp4",), "cool video"],
+            [("test/test_files/audio_sample.wav",), "cool audio"],
+            [("test/test_files/bus.png", "A bus"), "cool pic"],
         ]
         processed_multimodal_msg = [
-            (
+            [
                 {
-                    "name": "driving.mp4",
+                    "name": "video_sample.mp4",
                     "mime_type": "video/mp4",
                     "alt_text": None,
                     "data": None,
                     "is_file": True,
                 },
                 "cool video",
-            ),
-            (
+            ],
+            [
                 {
-                    "name": "cantina.wav",
+                    "name": "audio_sample.wav",
                     "mime_type": "audio/wav",
                     "alt_text": None,
                     "data": None,
                     "is_file": True,
                 },
                 "cool audio",
-            ),
-            (
+            ],
+            [
                 {
-                    "name": "lion.jpg",
-                    "mime_type": "image/jpeg",
-                    "alt_text": "A lion",
+                    "name": "bus.png",
+                    "mime_type": "image/png",
+                    "alt_text": "A bus",
                     "data": None,
                     "is_file": True,
                 },
                 "cool pic",
-            ),
+            ],
         ]
+        postprocessed_multimodal_msg = chatbot.postprocess(multimodal_msg)
+        postprocessed_multimodal_msg_base_names = []
+        for x, y in postprocessed_multimodal_msg:
+            if isinstance(x, dict):
+                x["name"] = os.path.basename(x["name"])
+                postprocessed_multimodal_msg_base_names.append([x, y])
+        assert postprocessed_multimodal_msg_base_names == processed_multimodal_msg
+
+        preprocessed_multimodal_msg = chatbot.preprocess(processed_multimodal_msg)
+        multimodal_msg_base_names = []
+        for x, y in multimodal_msg:
+            if isinstance(x, tuple):
+                if len(x) > 1:
+                    new_x = (os.path.basename(x[0]), x[1])
+                else:
+                    new_x = (os.path.basename(x[0]),)
+                multimodal_msg_base_names.append([new_x, y])
+        assert multimodal_msg_base_names == preprocessed_multimodal_msg
 
-        assert chatbot.postprocess(multimodal_msg) == processed_multimodal_msg
         assert chatbot.get_config() == {
             "value": [],
             "label": None,
@@ -1738,6 +1842,7 @@ class TestChatbot:
             "name": "chatbot",
             "visible": True,
             "elem_id": None,
+            "elem_classes": None,
             "style": {},
             "root_url": None,
             "selectable": False,
@@ -1754,6 +1859,7 @@ class TestJSON:
         assert js_output.get_config() == {
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "value": None,
             "show_label": True,
@@ -1808,6 +1914,7 @@ class TestHTML:
         assert {
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "value": "#Welcome onboard",
             "show_label": True,
@@ -1823,7 +1930,7 @@ class TestHTML:
         """
 
         def bold_text(text):
-            return "<strong>" + text + "</strong>"
+            return f"<strong>{text}</strong>"
 
         iface = gr.Interface(bold_text, "text", "html")
         assert iface("test") == "<strong>test</strong>"
@@ -1865,6 +1972,7 @@ class TestModel3D:
             "name": "model3d",
             "visible": True,
             "elem_id": None,
+            "elem_classes": None,
             "style": {},
         } == component.get_config()
 
@@ -1886,7 +1994,7 @@ class TestModel3D:
 class TestColorPicker:
     def test_component_functions(self):
         """
-        Preprocess, postprocess, serialize, tokenize, generate_sample, get_config
+        Preprocess, postprocess, serialize, tokenize, get_config
         """
         color_picker_input = gr.ColorPicker()
         assert color_picker_input.preprocess("#000000") == "#000000"
@@ -1903,12 +2011,12 @@ class TestColorPicker:
             "label": None,
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
             "name": "colorpicker",
         }
-        assert isinstance(color_picker_input.generate_sample(), str)
 
     def test_in_interface_as_input(self):
         """
@@ -1956,10 +2064,8 @@ class TestGallery:
         gallery = gr.Gallery()
         test_file_dir = Path(Path(__file__).parent, "test_files")
         data = [
-            gr.processing_utils.encode_file_to_base64(Path(test_file_dir, "bus.png")),
-            gr.processing_utils.encode_file_to_base64(
-                Path(test_file_dir, "cheetah1.jpg")
-            ),
+            client_utils.encode_file_to_base64(Path(test_file_dir, "bus.png")),
+            client_utils.encode_file_to_base64(Path(test_file_dir, "cheetah1.jpg")),
         ]
 
         with tempfile.TemporaryDirectory() as tmpdir:
@@ -2049,7 +2155,7 @@ def test_dataset_calls_as_example(*mocks):
             ]
         ],
     )
-    assert all([m.called for m in mocks])
+    assert all(m.called for m in mocks)
 
 
 cars = vega_datasets.data.cars()
@@ -2070,6 +2176,7 @@ class TestScatterPlot:
         assert gr.ScatterPlot().get_config() == {
             "caption": None,
             "elem_id": None,
+            "elem_classes": None,
             "interactive": None,
             "label": None,
             "name": "plot",
@@ -2090,7 +2197,7 @@ class TestScatterPlot:
             x_title="Horse",
         )
         output = plot.postprocess(cars)
-        assert sorted(list(output.keys())) == ["chart", "plot", "type"]
+        assert sorted(output.keys()) == ["chart", "plot", "type"]
         config = json.loads(output["plot"])
         assert config["encoding"]["x"]["field"] == "Horsepower"
         assert config["encoding"]["x"]["title"] == "Horse"
@@ -2111,7 +2218,7 @@ class TestScatterPlot:
             x="Horsepower", y="Miles_per_Gallon", tooltip="Name", interactive=False
         )
         output = plot.postprocess(cars)
-        assert sorted(list(output.keys())) == ["chart", "plot", "type"]
+        assert sorted(output.keys()) == ["chart", "plot", "type"]
         config = json.loads(output["plot"])
         assert "selection" not in config
 
@@ -2120,7 +2227,7 @@ class TestScatterPlot:
             x="Horsepower", y="Miles_per_Gallon", height=100, width=200
         )
         output = plot.postprocess(cars)
-        assert sorted(list(output.keys())) == ["chart", "plot", "type"]
+        assert sorted(output.keys()) == ["chart", "plot", "type"]
         config = json.loads(output["plot"])
         assert config["height"] == 100
         assert config["width"] == 200
@@ -2254,6 +2361,7 @@ class TestLinePlot:
         assert gr.LinePlot().get_config() == {
             "caption": None,
             "elem_id": None,
+            "elem_classes": None,
             "interactive": None,
             "label": None,
             "name": "plot",
@@ -2274,7 +2382,7 @@ class TestLinePlot:
             x_title="Trading Day",
         )
         output = plot.postprocess(stocks)
-        assert sorted(list(output.keys())) == ["chart", "plot", "type"]
+        assert sorted(output.keys()) == ["chart", "plot", "type"]
         config = json.loads(output["plot"])
         for layer in config["layer"]:
             assert layer["mark"]["type"] in ["line", "point"]
@@ -2289,7 +2397,7 @@ class TestLinePlot:
     def test_height_width(self):
         plot = gr.LinePlot(x="date", y="price", height=100, width=200)
         output = plot.postprocess(stocks)
-        assert sorted(list(output.keys())) == ["chart", "plot", "type"]
+        assert sorted(output.keys()) == ["chart", "plot", "type"]
         config = json.loads(output["plot"])
         assert config["height"] == 100
         assert config["width"] == 200
@@ -2417,6 +2525,7 @@ class TestBarPlot:
         assert gr.BarPlot().get_config() == {
             "caption": None,
             "elem_id": None,
+            "elem_classes": None,
             "interactive": None,
             "label": None,
             "name": "plot",
@@ -2437,7 +2546,7 @@ class TestBarPlot:
             x_title="Variable A",
         )
         output = plot.postprocess(simple)
-        assert sorted(list(output.keys())) == ["chart", "plot", "type"]
+        assert sorted(output.keys()) == ["chart", "plot", "type"]
         assert output["chart"] == "bar"
         config = json.loads(output["plot"])
         assert config["encoding"]["x"]["field"] == "a"
@@ -2452,7 +2561,7 @@ class TestBarPlot:
     def test_height_width(self):
         plot = gr.BarPlot(x="a", y="b", height=100, width=200)
         output = plot.postprocess(simple)
-        assert sorted(list(output.keys())) == ["chart", "plot", "type"]
+        assert sorted(output.keys()) == ["chart", "plot", "type"]
         config = json.loads(output["plot"])
         assert config["height"] == 100
         assert config["width"] == 200
@@ -2566,7 +2675,15 @@ class TestCode:
         assert code.preprocess("# hello friends") == "# hello friends"
         assert code.preprocess("def fn(a):\n  return a") == "def fn(a):\n  return a"
 
-        assert code.postprocess("def fn(a):\n  return a") == "def fn(a):\n  return a"
+        assert (
+            code.postprocess(
+                """
+            def fn(a):
+                return a
+            """
+            )
+            == "def fn(a):\n    return a"
+        )
 
         test_file_dir = Path(Path(__file__).parent, "test_files")
         path = str(Path(test_file_dir, "test_label_json.json"))
@@ -2580,13 +2697,95 @@ class TestCode:
         assert code.get_config() == {
             "value": None,
             "language": None,
+            "lines": 5,
             "name": "code",
             "show_label": True,
             "label": None,
             "style": {},
             "elem_id": None,
+            "elem_classes": None,
             "visible": True,
             "interactive": None,
             "root_url": None,
         }
-        assert isinstance(code.generate_sample(), str)
+
+
+class TestTempFileManagement:
+    def test_hash_file(self):
+        temp_file_manager = gr.File()
+        h1 = temp_file_manager.hash_file("gradio/test_data/cheetah1.jpg")
+        h2 = temp_file_manager.hash_file("gradio/test_data/cheetah1-copy.jpg")
+        h3 = temp_file_manager.hash_file("gradio/test_data/cheetah2.jpg")
+        assert h1 == h2
+        assert h1 != h3
+
+    @patch("shutil.copy2")
+    def test_make_temp_copy_if_needed(self, mock_copy):
+        temp_file_manager = gr.File()
+
+        f = temp_file_manager.make_temp_copy_if_needed("gradio/test_data/cheetah1.jpg")
+        try:  # Delete if already exists from before this test
+            os.remove(f)
+        except OSError:
+            pass
+
+        f = temp_file_manager.make_temp_copy_if_needed("gradio/test_data/cheetah1.jpg")
+        assert mock_copy.called
+        assert len(temp_file_manager.temp_files) == 1
+        assert Path(f).name == "cheetah1.jpg"
+
+        f = temp_file_manager.make_temp_copy_if_needed("gradio/test_data/cheetah1.jpg")
+        assert len(temp_file_manager.temp_files) == 1
+
+        f = temp_file_manager.make_temp_copy_if_needed(
+            "gradio/test_data/cheetah1-copy.jpg"
+        )
+        assert len(temp_file_manager.temp_files) == 2
+        assert Path(f).name == "cheetah1-copy.jpg"
+
+    def test_base64_to_temp_file_if_needed(self):
+        temp_file_manager = gr.File()
+
+        base64_file_1 = media_data.BASE64_IMAGE
+        base64_file_2 = media_data.BASE64_AUDIO["data"]
+
+        f = temp_file_manager.base64_to_temp_file_if_needed(base64_file_1)
+        try:  # Delete if already exists from before this test
+            os.remove(f)
+        except OSError:
+            pass
+
+        f = temp_file_manager.base64_to_temp_file_if_needed(base64_file_1)
+        assert len(temp_file_manager.temp_files) == 1
+
+        f = temp_file_manager.base64_to_temp_file_if_needed(base64_file_1)
+        assert len(temp_file_manager.temp_files) == 1
+
+        f = temp_file_manager.base64_to_temp_file_if_needed(base64_file_2)
+        assert len(temp_file_manager.temp_files) == 2
+
+        for file in temp_file_manager.temp_files:
+            os.remove(file)
+
+    @pytest.mark.flaky
+    @patch("shutil.copyfileobj")
+    def test_download_temp_copy_if_needed(self, mock_copy):
+        temp_file_manager = gr.File()
+        url1 = "https://raw.githubusercontent.com/gradio-app/gradio/main/gradio/test_data/test_image.png"
+        url2 = "https://raw.githubusercontent.com/gradio-app/gradio/main/gradio/test_data/cheetah1.jpg"
+
+        f = temp_file_manager.download_temp_copy_if_needed(url1)
+        try:  # Delete if already exists from before this test
+            os.remove(f)
+        except OSError:
+            pass
+
+        f = temp_file_manager.download_temp_copy_if_needed(url1)
+        assert mock_copy.called
+        assert len(temp_file_manager.temp_files) == 1
+
+        f = temp_file_manager.download_temp_copy_if_needed(url1)
+        assert len(temp_file_manager.temp_files) == 1
+
+        f = temp_file_manager.download_temp_copy_if_needed(url2)
+        assert len(temp_file_manager.temp_files) == 2
