@@ -5,10 +5,8 @@ from contextlib import contextmanager
 from functools import partial
 from string import capwords
 
-import mlflow
 import pytest
 import requests
-import wandb
 from fastapi.testclient import TestClient
 
 import gradio
@@ -54,7 +52,7 @@ class TestInterface:
 
     def test_partial_functions(self):
         def greet(name, formatter):
-            return formatter("Hello " + name + "!")
+            return formatter(f"Hello {name}!")
 
         greet_upper_case = partial(greet, formatter=capwords)
         demo = Interface(fn=greet_upper_case, inputs="text", outputs="text")
@@ -84,7 +82,7 @@ class TestInterface:
         )
         interface = Interface(lambda x: 3 * x, "number", "number", examples=path)
         dataset_check = any(
-            [c["type"] == "dataset" for c in interface.get_config_file()["components"]]
+            c["type"] == "dataset" for c in interface.get_config_file()["components"]
         )
         assert dataset_check
 
@@ -119,7 +117,9 @@ class TestInterface:
         interface.close()
 
     def test_interface_representation(self):
-        prediction_fn = lambda x: x
+        def prediction_fn(x):
+            return x
+
         prediction_fn.__name__ = "prediction_fn"
         repr = str(Interface(prediction_fn, "textbox", "label")).split("\n")
         assert prediction_fn.__name__ in repr[0]
@@ -160,63 +160,6 @@ class TestInterface:
         assert mock_display.call_count == 2
         interface.close()
 
-    @mock.patch("comet_ml.Experiment")
-    def test_integration_comet(self, mock_experiment):
-        experiment = mock_experiment()
-        experiment.log_text = mock.MagicMock()
-        experiment.log_other = mock.MagicMock()
-        interface = Interface(lambda x: x, "textbox", "label")
-        interface.launch(prevent_thread_lock=True)
-        interface.integrate(comet_ml=experiment)
-        experiment.log_text.assert_called_with("gradio: " + interface.local_url)
-        interface.share_url = "tmp"  # used to avoid creating real share links.
-        interface.integrate(comet_ml=experiment)
-        experiment.log_text.assert_called_with("gradio: " + interface.share_url)
-        assert experiment.log_other.call_count == 2
-        interface.share_url = None
-        interface.close()
-
-    def test_integration_mlflow(self):
-        mlflow.log_param = mock.MagicMock()
-        interface = Interface(lambda x: x, "textbox", "label")
-        interface.launch(prevent_thread_lock=True)
-        interface.integrate(mlflow=mlflow)
-        mlflow.log_param.assert_called_with(
-            "Gradio Interface Local Link", interface.local_url
-        )
-        interface.share_url = "tmp"  # used to avoid creating real share links.
-        interface.integrate(mlflow=mlflow)
-        mlflow.log_param.assert_called_with(
-            "Gradio Interface Share Link", interface.share_url
-        )
-        interface.share_url = None
-        interface.close()
-
-    def test_integration_wandb(self):
-        with captured_output() as (out, err):
-            wandb.log = mock.MagicMock()
-            wandb.Html = mock.MagicMock()
-            interface = Interface(lambda x: x, "textbox", "label")
-            interface.width = 500
-            interface.height = 500
-            interface.integrate(wandb=wandb)
-
-            assert (
-                out.getvalue().strip()
-                == "The WandB integration requires you to `launch(share=True)` first."
-            )
-            interface.share_url = "tmp"
-            interface.integrate(wandb=wandb)
-            wandb.log.assert_called_once()
-
-    @mock.patch("requests.post")
-    def test_integration_analytics(self, mock_post):
-        mlflow.log_param = mock.MagicMock()
-        interface = Interface(lambda x: x, "textbox", "label")
-        interface.analytics_enabled = True
-        interface.integrate(mlflow=mlflow)
-        mock_post.assert_called_once()
-
 
 class TestTabbedInterface:
     def test_tabbed_interface_config_matches_manual_tab(self):
